/*
  $Id: ActionManager.java,v 1.2 2006/09/08 14:37:41 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package action;
import java.util.*;
import java.util.regex.*;
import java.lang.reflect.*;
import com.apple.cocoa.foundation.*;

/*
  GUIを除き、アクションについての全ての事柄を管理するクラスです。
*/

public class ActionManager {
    private static ActionManager _shared;
    protected static TreeMap action_classes; // {アクション名 => BottleAction.class}

    Vector installed_actions; // [BottleAction]
    
    static {
		action_classes = new TreeMap();
		action_classes.put(
			action.drop.Drop.getName(), action.drop.Drop.class);	    
		action_classes.put(
			action.strange.StrangeFilter.getName(), action.strange.StrangeFilter.class);
		action_classes.put(
			action.modghost.ModGhost.getName(), action.modghost.ModGhost.class);
    }

    public static ActionManager sharedManager() {
		if (_shared == null) {
			_shared = new ActionManager();
		}
		return _shared;
    }

    protected ActionManager() {
		this.installed_actions = new Vector();
		loadActions();
    }

    public Vector getInstalledActions() {
		// インストールされてゐるアクションのインスタンスのリストを返します。變更しないで下さい。
		return installed_actions;
    }

    public BottleAction getNthAction(int n) {
		// n番目のアクションを返す。
		return (BottleAction)installed_actions.get(n);
    }

    public void installAction(BottleAction action) {
		installAction(action,-1);
    }
    public void installAction(BottleAction action,int pos) {
		// posで指定された場所に、このアクションをインストールする。
		// posが0からinstalled_actions.size()-1までの範圍に入つてゐなければ、末尾に追加する。
		if (pos >= 0 && pos <= installed_actions.size()-1) {
			installed_actions.add(pos,action);
		}
		else {
			installed_actions.add(action);
		}
    }

    public static TreeMap getActionClassList() {
		// 登録されたBottleActionクラスのアクション名とClassとが對應するTreeMapを返します。變更しないで下さい。
		return action_classes;
    }

    public static Vector getActionNameList() {
		// 登録されたBottleActionクラスのアクション名のリストを返します。
		Vector result = new Vector();
		for (Iterator ite = action_classes.keySet().iterator(); ite.hasNext(); ) {
			Class action = (Class)action_classes.get((String)ite.next());
			String name = getName(action);
			if (name != null) {
				result.add(name);
			}
		}
		return result;
    }

    public static boolean isAction(Class c) {
		// cがBottleActionを繼承してゐれば眞を返します。
		return BottleAction.class.isAssignableFrom(c);
    }

    public static String getName(BottleAction action) {
		return getName(action.getClass());
    }
    public static String getName(Class c) {
		// BottleActionであるクラスcのアクション名を返します。
		// 取得出來なければnullを返します。
		if (isAction(c)) {
			try {
				Method getName = c.getMethod("getName",null);
				String name = (String)getName.invoke(null,null);
				if (name != null) {
					return name;
				}
			} catch (Exception e) {}
		}
		return null;
    }

    public static String getShortDescription(String name) {
		Class c = getActionClass(name);
		if (c != null) {
			return getShortDescription(c);
		}
		return "";
    }
    public static String getShortDescription(Class c) {
		// BottleActionであるクラスcの短い説明を返します。
		// 取得出來なければ空文字列を返します。
		if (isAction(c)) {
			return NSBundle.mainBundle().localizedStringForKey(
				c.getName()+".description.short","","InfoPlist");
		}
		else {
			return "";
		}
    }

    public static Class getActionClass(String name) {
		// 指定された名前を持つBottleActionクラスを返します。
		return (Class)action_classes.get(name);
    }

    public void removeActions(Iterator ite) {
		// ite: Iterator<Integer> 削除すべきアクションの番號
	
		for (; ite.hasNext(); ) {
			int n = ((Integer)ite.next()).intValue();
			installed_actions.set(n, null);
		}

		for (int i = 0; i < installed_actions.size(); i++) {
			if (installed_actions.get(i) == null) {
				installed_actions.remove(i);
				i--;
			}
		}
    }

    public void moveUpper(Iterator iter_items) {
		/*
		  iter_items: <Integer>要素番號
	  
		  移動は次のやうに行ふ。
		  1. 移動すべき項目が全て一つ上にずれてゐて、
		  その他の部分はnullであるやうな配列を作る。
		  2. 移動しなかった項目を、nullになつてゐる場所に
		  上から一つずつ置いて行く。
		*/
		moveActions(iter_items, -1);
    }
    public void moveDowner(Iterator iter_items) {
		moveActions(iter_items, +1);
    }
    public void moveActions(Iterator iter_items, int delta) {
		BottleAction[] array = new BottleAction[
			installed_actions.size()];
		boolean[] is_put_already = new boolean[
			installed_actions.size()]; // 移動すべき項目であればtrue。
	
		for (; iter_items.hasNext(); ) {
			int idx = ((Integer)iter_items.next()).intValue();
			array[idx + delta] = (BottleAction)installed_actions.get(idx);
			is_put_already[idx] = true;
		}
	
		for (int src = 0, dest = 0;
			 src < installed_actions.size();
			 src++) {
			// 移動すべき項目は、既に移動が終わつてゐる。
			if (!is_put_already[src]) {
				// 次のnullの場所を探す。探すのはdest以降。
				for (;; dest++) {
					if (array[dest] == null) {
						break;
					}
				}
				array[dest] = (BottleAction)installed_actions.get(src);
			}
		}

		// 配列が完成した。これをVectorにコピー。
		Vector new_vec = new Vector(array.length);
		for (int i = 0; i < array.length; i++) {
			new_vec.add(array[i]);
		}
		installed_actions = new_vec;
    }

    public void loadActions() {
		installed_actions.clear();
	
		// defaultsからアクションのmementoを取出し、それを元に復元します。
		NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
		synchronized (defaults) {
			NSArray root = defaults.arrayForKey("actions");
			if (root == null) {
				return;
			}

			for (Enumeration enu = root.objectEnumerator();
				 enu.hasMoreElements(); ) {
				try {
					NSDictionary action_info = (NSDictionary)enu.nextElement();
		    
					// この名を持つクラスを取得。無ければ次へ。
					String class_name = (String)action_info.objectForKey("class-name");
					Class action_class = Class.forName(class_name);

					// mementoを取得し、變換する。
					NSDictionary cf_memento = (NSDictionary)action_info.objectForKey("memento");
					Map memento = convertMemento(cf_memento);

					// このmementoを用ゐてアクションのオブジェクトを生成する。
					Constructor con =
						action_class.getConstructor(new Class[] {Map.class});
					BottleAction act =
						(BottleAction)con.newInstance(new Object[] {memento});

					installed_actions.add(act);
				}
				catch (Exception e) {
					System.out.println("Couldn't reconstruct some action from a memento.");
					e.printStackTrace();
				}
			}
		}
    }
    public static Map convertMemento(NSDictionary dic) throws Exception {
		Map map = new Hashtable();
		for (Enumeration enu = dic.keyEnumerator(); enu.hasMoreElements(); ) {
			String key = (String)enu.nextElement();
			Object val_obj = dic.objectForKey(key);

			if (val_obj instanceof NSArray) {
				map.put(key, convertMemento((NSArray)val_obj));
			}
			else if (val_obj instanceof NSDictionary) {
				map.put(key, convertMemento((NSDictionary)val_obj));
			}
			else {
				map.put(key, val_obj);
			}
		}
		return map;
    }
    public static List convertMemento(NSArray array) throws Exception {
		List list = new Vector();
		for (Enumeration enu = array.objectEnumerator(); enu.hasMoreElements(); ) {
			Object val_obj = enu.nextElement();

			if (val_obj instanceof NSArray) {
				list.add(convertMemento((NSArray)val_obj));
			}
			else if (val_obj instanceof NSDictionary) {
				list.add(convertMemento((NSDictionary)val_obj));
			}
			else {
				list.add(val_obj);
			}
		}
		return list;
    }

    public void saveActions() {
		// 各アクションのmementoを取得し、その形式をチェックしつつ、一つのNSArrayに纏めます。
		// さうして作つたNSArrayをdefaultsに保存します。
		NSMutableArray root = new NSMutableArray();
		for (Iterator ite = installed_actions.iterator(); ite.hasNext(); ) {
			BottleAction act = (BottleAction)ite.next();
			Map memento = act.getMemento();
			try {
				NSMutableDictionary action_info = new NSMutableDictionary();
				// class-name: (String)このアクションのクラス名
				action_info.setObjectForKey(
					act.getClass().getName(), "class-name");
				// memento: (NSDictionary)言ふまでも無い
				action_info.setObjectForKey(
					convertMemento(memento), "memento");
				root.addObject(action_info);
			}
			catch (Exception e) {
				System.out.println(
					"Something's strange of a memento of "+getName(act)+": "+memento);
				e.printStackTrace();
			}
		}

		NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
		synchronized (defaults) {
			defaults.setObjectForKey(root, "actions");
			defaults.synchronize();
		}
    }
    protected static NSDictionary convertMemento(Map memento) throws Exception {
		NSMutableDictionary dic = new NSMutableDictionary();
	
		for (Iterator ite_key = memento.keySet().iterator(); ite_key.hasNext(); ) {
			Object key_obj = ite_key.next();
	    
			if (key_obj instanceof String) {
				String key = (String)key_obj;
				Object val_obj = memento.get(key);
		
				if (val_obj instanceof String ||
					val_obj instanceof Number) {
					dic.setObjectForKey(val_obj, key);
				}
				else if (val_obj instanceof byte[]) {
					dic.setObjectForKey(new NSData((byte[])val_obj), key);
				}
				else if (val_obj instanceof List) {
					dic.setObjectForKey(convertMemento((List)val_obj), key);
				}
				else if (val_obj instanceof Map) {
					dic.setObjectForKey(convertMemento((Map)val_obj), key);
				}
				else {
					throw new Exception(val_obj.getClass().getName()+" used for a value: "+val_obj);
				}
			}
			else {
				throw new Exception(key_obj.getClass().getName()+" used for a key: "+key_obj);
			}
		}

		return dic;
    }
    protected static NSArray convertMemento(List memento) throws Exception {
		NSMutableArray array = new NSMutableArray();
	
		for (Iterator ite_val = memento.iterator(); ite_val.hasNext(); ) {
			Object val_obj = ite_val.next();

			if (val_obj instanceof String ||
				val_obj instanceof Number) {
				array.addObject(val_obj);
			}
			else if (val_obj instanceof byte[]) {
				array.addObject(new NSData((byte[])val_obj));
			}
			else if (val_obj instanceof List) {
				array.addObject(convertMemento((List)val_obj));
			}
			else if (val_obj instanceof Map) {
				array.addObject(convertMemento((Map)val_obj));
			}
			else {
				throw new Exception(val_obj.getClass().getName()+" used for a value: "+val_obj);
			}
		}

		return array;
    }

    static Pattern pat_backslash = Pattern.compile("\\\\");
    public ActionReply doAction(
		boolean replaying,
		String channel,
		String ifghost,
		String script,
		String slpp_server) {
	
		// スクリプトのバックスラッシュは圓記號に置換。
		if (script.indexOf('\\') != -1) {
			script = pat_backslash.matcher(script).replaceAll("\u00a5");
		}

		// リクエストを構築
		ActionRequest req = new ActionRequest(
			replaying, channel, ifghost, script, slpp_server);

		// 順々にフィルタを通して行く。
		ActionReply result = new ActionReply();
		for (int i = 0; i < installed_actions.size(); i++) {
			BottleAction act = (BottleAction)installed_actions.get(i);
	    
			req.setRemarks(result.getRemarks());
			ActionReply rep = act.doAction(req);

			// このrepのコマンドと註釋を、それぞれresultにマージ。
			result.setCommands(rep.getCommands());
			result.setRemarks(rep.getRemarks());

			// stop-actionが設定されてゐたら、ここで終はり。
			if (result.getStopAction()) {
				break;
			}
		}

		return result;
    }
}
