/*
  $Id: BottleAction.java,v 1.2 2006/09/08 14:37:41 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package action;
import java.util.*;
import java.lang.reflect.*;

/*
  アクションの基本的な性質を表すインターフェイスです。
  全てのアクションは、このインターフェイスを繼承してゐる必要があります。
*/

public interface BottleAction {

    // public Foo();
    // public Foo(Map memento);
    // 全てのBottleActionは、引數を取らないコンストラクタと、引數Mapを取るコンストラクタの兩者を
    // 定義してゐる必要があります。mementoはアクションの設定を保存する爲のオブジェクトで、
    // 保存された設定を引繼ぐ場合にこちらのコンストラクタが使はれます。

    public void configure(Object callback_target,Method callback_ok,Method callback_cancel);
    // callback_target: OKまたはCancelが押された場合に呼ばれるオブジェクト。省略可能。
    // callback_ok: OKが押された場合に呼ばれるメソッド。引数はDrop。省略可能。
    // callback_cancel: Cancelが押された場合に呼ばれるメソッド。引数はDrop。省略可能。

    public String getConfiguration();
    // このアクションの設定を文字列で表したものを返して下さい。
    // 人間が讀む爲のものであり、フォーマットはどの樣なものでも構ひません。

    public String getStatus();
    // このアクションの現在の状態を文字列で表したものを返して下さい。
    // 例へば子プロセスを立てて通信するやうなアクションは、そのプロセスが死んでゐたら
    // 死んでゐる旨を表示するべきです。
    // これも人間が讀む爲のものであり、フォーマットはどの樣なものでも構ひません。

    public void destruct();
    // このアクションが最早必要とされなくなつた時に、ModuleManagerによつて呼ばれるメソッド。
    // destruct後に再びこのアクションが使はれる事はありませんが、もし何らかのメソッドが呼ばれるやうな事があれば
    // アクションはRuntimeExceptionを投げるべきでせう。

    public ActionReply doAction(ActionRequest req);
    // アクションの中心部で、ActionRequestを見て、最終的な動作を決めます。
    // 一本の瓶に対し、複數のアクションが別々の指示をした場合、最後に指示されたものが使はれます。
    // (それを避ける爲にコマンドstop-actionが存在します。)
    // このメソッドは出來る限りの速度で處理を終える事が望ましく、リプレイ時には特にさうです。

    public Map getMemento();
    // オブジェクトの内部状態を保存する爲に必要な全ての情報を持つたMapを返して下さい。
    // キーとして許されるものはStringのみであり、値として許されるものは次のものに限られます。
    // String, Number, byte[], List, Map
    // 異常なmementoを返した場合は、データの保存が行はれません。
    
    // public static String getName();
    // このアクションの名前。必ず定義して、nullでない文字列を返して下さい。
    // アクションリストのType欄や、アクション追加畫面でのポップアップリストに現れます。
    // 呼ばれた時には、常に同じ値を返して下さい。

}
