/*
  $Id: Drop.java,v 1.2 2006/09/08 14:37:41 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package action.drop;
import action.*;
import action.drop.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.regex.*;

public class Drop implements BottleAction {
    WindowController wc;

    String ifghost_regex;
    String script_regex;

    Pattern ifghost_pat; // ifghost_regexをコンパイルしたもの。キャッシュ。
    Pattern script_pat; // 同樣。

    boolean drop_from_log;

    public Drop() {
		this.wc = new WindowController(this);
	
		this.ifghost_regex = "";
		this.ifghost_pat = null;

		this.script_regex = "";
		this.script_pat = null;
	
		this.drop_from_log = false;
    }

    public Drop(Map memento) {
		this();
		this.ifghost_regex =
			(String)defined_or(memento.get("ifghost-regex"), "");
		this.script_regex =
			(String)defined_or(memento.get("script-regex"), "");
		this.drop_from_log =
			((Number)defined_or(memento.get("drop-also-from-log"),
								new Integer(0))).intValue() != 0;
    }

    protected Object defined_or(Object obj, Object def) {
		return obj == null ? def : obj;
    }

    public String getIfghostRegex() {
		return ifghost_regex;
    }

    protected void setIfghostRegex(String regex) {
		ifghost_regex = regex;
		ifghost_pat = null;
    }

    public String getScriptRegex() {
		return script_regex;
    }

    protected void setScriptRegex(String regex) {
		script_regex = regex;
		script_pat = null;
    }

    public boolean getDropFromLog() {
		return drop_from_log;
    }

    protected void setDropFromLog(boolean bool) {
		drop_from_log = bool;
    }

    public void configure(Object callback_target,Method callback_ok,Method callback_cancel) {
		wc.configure(callback_target,callback_ok,callback_cancel);
    }

    public String getConfiguration() {
		StringBuffer buf = new StringBuffer();
	
		if (ifghost_regex.length() > 0) {
			buf.append("{IfGhost: "+ifghost_regex+"}");
		}

		if (script_regex.length() > 0) {
			if (buf.length() > 0) {
				buf.append(' ');
			}
			buf.append("{Script: "+script_regex+"}");
		}

		if (drop_from_log) {
			buf.append(" (drop log)");
		}
	
		return buf.toString();
    }

    public String getStatus() {
		return "ok";
    }

    public void destruct() {
    }

    public ActionReply doAction(ActionRequest req) {
		// IfGhostが正規表現ifghost_regexにマッチすればdropを設定する。
		ActionReply rep = new ActionReply();
	
		if (ifghost_pat == null) {
			try {
				ifghost_pat = Pattern.compile(ifghost_regex);
			}
			catch (Exception e) {
				throw new RuntimeException(e);
			}
		}

		if (script_pat == null) {
			try {
				script_pat = Pattern.compile(script_regex);
			}
			catch (Exception e) {
				throw new RuntimeException(e);
			}
		}

		if (ifghost_pat.matcher(req.getIfGhost()).find() &&
			script_pat.matcher(req.getScript()).find()) {
			// 破棄設定。
			rep.setDrop(drop_from_log);
		}

		return rep;
    }

    public Map getMemento() {
		Hashtable memento = new Hashtable();
		memento.put("ifghost-regex", ifghost_regex);
		memento.put("script-regex", script_regex);
		memento.put("drop-also-from-log", new Integer(drop_from_log ? 1 : 0));
		return memento;
    }

    public static String getName() {
		return "Drop";
    }
}
