/*
  $Id: WindowController.java,v 1.2 2006/09/08 14:37:41 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package action.drop;
import action.drop.*;
import java.lang.reflect.*;
import java.util.regex.*;
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

/*
  このクラスはifghost及びscriptのDelegateに指定されてゐる。
*/

public class WindowController extends NSWindowController {
    public static final String MSG_REGEX_ERROR = "illegal regex";
    
    Drop drop;
    Object config_callback_target;
    Method config_callback_ok;
    Method config_callback_cancel;
    
    NSTextField ifghost; /* IBOutlet */
    NSTextField ifghost_regex_error; /* IBOutlet */
    NSTextField script; /* IBOutlet */
    NSTextField script_regex_error; /* IBOutlet */
    NSButton drop_from_log; /* IBOutlet */
    NSButton ok; /* IBOutlet */

    public WindowController(Drop drop) {
		super("ActionDrop");
		loadWindow();
		this.drop = drop;
    }

    public void configure(Object callback_target,Method callback_ok,Method callback_cancel) {
		// callback_target: OKまたはCancelが押された場合に呼ばれるオブジェクト。省略可能。
		// callback_ok: OKが押された場合に呼ばれるメソッド。引数はDrop。省略可能。
		// callback_cancel: Cancelが押された場合に呼ばれるメソッド。引数はDrop。省略可能。
		config_callback_target = callback_target;
		config_callback_ok = callback_ok;
		config_callback_cancel = callback_cancel;

		drop_from_log.setIntValue(drop.getDropFromLog() ? 1 : 0);
		ifghost.setStringValue(drop.getIfghostRegex());
		ifghost_regex_error.setStringValue("");
		script.setStringValue(drop.getScriptRegex());
		script_regex_error.setStringValue("");

		controlTextDidChange(null);
		showWindow(this);
    }

    public void cancel(Object sender) { /* IBAction */
		window().close();

		if (config_callback_target != null &&
			config_callback_cancel != null) {
			try {
				config_callback_cancel.invoke(config_callback_target,new Object[] {drop});
			} catch (Exception e) {}
		}
    }

    public void ok(Object sender) { /* IBAction */
		drop.setIfghostRegex(ifghost.stringValue());
		drop.setScriptRegex(script.stringValue());
		drop.setDropFromLog(drop_from_log.intValue() != 0);
	
		window().close();

		if (config_callback_target != null &&
			config_callback_ok != null) {
			try {
				config_callback_ok.invoke(config_callback_target,new Object[] {drop});
			} catch (Exception e) {}
		}
    }

    // Delegate
    public void controlTextDidChange(NSNotification aNotification) {
		// IfGhost正規表現及びScript正規表現の雙方が空であれば、OKボタンは使用不可能。
		// 何れかの正規表現がJava正規表現として不正であれば、OKボタンは使用不可能。
		String ifghost_regex = ifghost.stringValue();
		String script_regex = script.stringValue();

		boolean bad_regex = false;
	
		if (ifghost_regex.length() == 0 &&
			script_regex.length() == 0) {
	    
			bad_regex = true;
		}

		try {
			Pattern.compile(ifghost_regex);
			// 問題無い
			if (ifghost_regex_error.stringValue().length() > 0) {
				ifghost_regex_error.setStringValue("");
			}
		}
		catch (Exception e) {
			// 駄目
			if (ifghost_regex_error.stringValue().length() == 0) {
				ifghost_regex_error.setStringValue(MSG_REGEX_ERROR);
			}
			bad_regex = true;
		}

		try {
			Pattern.compile(script_regex);
			// 問題無い
			if (script_regex_error.stringValue().length() > 0) {
				script_regex_error.setStringValue("");
			}
		}
		catch (Exception e) {
			// 駄目
			if (script_regex_error.stringValue().length() == 0) {
				script_regex_error.setStringValue(MSG_REGEX_ERROR);
			}
			bad_regex = true;
		}

		ok.setEnabled(!bad_regex);
    }
}
