/*
  $Id: ModGhost.java,v 1.2 2006/09/08 14:37:41 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package action.modghost;
import action.*;
import action.modghost.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.regex.*;

public class ModGhost implements BottleAction {
    WindowController wc;

    boolean replace_all;
    String ifghost_regex;
    String replace_string;

    Pattern ifghost_pat; // ifghost_regexをコンパイルしたキャッシュ。

    public ModGhost() {
		this.wc = new WindowController(this);

		this.replace_all = false;
		this.ifghost_regex = "";
		this.replace_string = "";

		this.ifghost_pat = null;
    }

    public ModGhost(Map memento) {
		this();
		this.replace_all =
			((Number)defined_or(memento.get("replace-all"),
								new Integer(0))).intValue() != 0;
		this.ifghost_regex =
			(String)defined_or(memento.get("ifghost-regex"), "");
		this.replace_string =
			(String)defined_or(memento.get("replace-string"), "");
    }

    protected Object defined_or(Object obj, Object def) {
		return obj == null ? def : obj;
    }

    public boolean getReplaceAll() {
		return replace_all;
    }

    public void setReplaceAll(boolean bool) {
		replace_all = bool;
    }

    public String getIfGhostRegex() {
		return ifghost_regex;
    }

    public void setIfGhostRegex(String regex) {
		ifghost_regex = regex;
		ifghost_pat = null;
    }

    public String getReplaceString() {
		return replace_string;
    }

    public void setReplaceString(String str) {
		replace_string = str;
    }

    public void configure(Object callback_target,Method callback_ok,Method callback_cancel) {
		wc.configure(callback_target,callback_ok,callback_cancel);
    }

    public String getConfiguration() {
		StringBuffer buf = new StringBuffer();
		buf.append("s{").append(ifghost_regex).append('}');
		buf.append('{').append(replace_string).append('}');
		if (replace_all) {
			buf.append('g');
		}
		return buf.toString();
    }

    public String getStatus() {
		return "ok";
    }

    public void destruct() {
    }

    public ActionReply doAction(ActionRequest req) {
		ActionReply rep = new ActionReply();
	
		// IfGhostが無ければ何もしない。
		String original = req.getIfGhost();
		if (original != null) {
			if (ifghost_pat == null) {
				try {
					ifghost_pat = Pattern.compile(ifghost_regex);
				}
				catch (Exception e) {
					throw new RuntimeException(e);
				}
			}

			Matcher m = ifghost_pat.matcher(original);
			String modified = null;
			if (m.find()) {
				if (replace_all) {
					try {
						modified = m.replaceAll(replace_string);
					}
					catch (Exception e) {
					}
				}
				else {
					try {
						StringBuffer buf = new StringBuffer();
						m.appendReplacement(buf, replace_string);
						m.appendTail(buf);
						modified = buf.toString();
					}
					catch (Exception e) {
					}
				}
			}

			if (modified != null && modified.length() > 0) {
				// IfGhost變更
				rep.setIfGhostForSSTPServer(modified);
			}
		}
	
		return rep;
    }

    public Map getMemento() {
		Map memento = new Hashtable();
		memento.put("replace-all", new Integer(replace_all ? 1 : 0));
		memento.put("ifghost-regex", ifghost_regex);
		memento.put("replace-string", replace_string);
		return memento;
    }

    public static String getName() {
		return "Modify IfGhost";
    }
}