/*
  $Id: WindowController.java,v 1.2 2006/09/08 14:37:41 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package action.modghost;
import action.modghost.*;
import java.lang.reflect.*;
import java.util.regex.*;
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

/*
  このクラスはifghost_regexのDelegateに設定されてゐる。
*/

public class WindowController extends NSWindowController {
    public static final String MSG_REGEX_ERROR = "illegal regex";

    ModGhost mod;
    Object config_callback_target;
    Method config_callback_ok;
    Method config_callback_cancel;

    public NSTextField ifghost_regex; /* IBOutlet */
    public NSTextField ifghost_regex_error; /* IBOutlet */
    public NSTextField replace; /* IBOutlet */
    public NSButton replace_all; /* IBOutlet */
    public NSButton ok; /* IBOutlet */

    public WindowController(ModGhost mod) {
		super("ActionModifyIfGhost");
		loadWindow();
		this.mod = mod;
    }

    public void configure(Object callback_target,Method callback_ok,Method callback_cancel) {
		// callback_target: OKまたはCancelが押された場合に呼ばれるオブジェクト。省略可能。
		// callback_ok: OKが押された場合に呼ばれるメソッド。引数はDrop。省略可能。
		// callback_cancel: Cancelが押された場合に呼ばれるメソッド。引数はDrop。省略可能。
		config_callback_target = callback_target;
		config_callback_ok = callback_ok;
		config_callback_cancel = callback_cancel;

		replace_all.setIntValue(mod.getReplaceAll() ? 1 : 0);
		ifghost_regex.setStringValue(mod.getIfGhostRegex());
		ifghost_regex_error.setStringValue("");
		replace.setStringValue(mod.getReplaceString());

		controlTextDidChange(null);
		showWindow(this);
    }

    public void cancel(Object sender) { /* IBAction */
		window().close();

		if (config_callback_target != null &&
			config_callback_cancel != null) {
			try {
				config_callback_cancel.invoke(
					config_callback_target,new Object[] {mod});
			} catch (Exception e) {}
		}
    }

    public void ok(Object sender) { /* IBAction */
		mod.setReplaceAll(replace_all.intValue() != 0);
		mod.setIfGhostRegex(ifghost_regex.stringValue());
		mod.setReplaceString(replace.stringValue());

		window().close();

		if (config_callback_target != null &&
			config_callback_ok != null) {
			try {
				config_callback_ok.invoke(
					config_callback_target,new Object[] {mod});
			} catch (Exception e) {}
		}
    }

    // Delegate
    public void controlTextDidChange(NSNotification aNotification) {
		// IfGhost正規表現が空、若しくは正規表現として不正であれば、OKボタンは使用不可。
		// 置換文字列は空であつても構はない。
		String regex_ifghost = ifghost_regex.stringValue();

		boolean submit_ok = true;
		if (regex_ifghost.length() == 0) {
			submit_ok = false;
		}

		try {
			Pattern.compile(regex_ifghost);
			// 問題無い
			if (ifghost_regex_error.stringValue().length() > 0) {
				ifghost_regex_error.setStringValue("");
			}
		}
		catch (Exception e) {
			// 駄目
			if (ifghost_regex_error.stringValue().length() == 0) {
				ifghost_regex_error.setStringValue(MSG_REGEX_ERROR);
			}
			submit_ok = false;
		}

		ok.setEnabled(submit_ok);
    }
}
