/*
  $Id: StrangeFilter.java,v 1.2 2006/09/08 14:37:41 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package action.strange;
import action.*;
import action.strange.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.regex.*;

public class StrangeFilter implements BottleAction {
    // 0からの連番であり、NUM_CHECK_ITEMSは必ず正しい値が設定されてゐる必要があります。
    public static final int CHK_NONTERMINATE = 0;
    public static final int CHK_SCOPE = 1;
    public static final int CHK_SURFACE = 2;
    public static final int NUM_CHECK_ITEMS = 3;
    public static final String[] human_readable_check_type = {
		"\u00a5t",
		"scope",
		"surface",
		// "wait before scope",
    };
    public static final String[] for_memento_check_type = {
		"check-nonterminate-tag",
		"check-scope-tag",
		"check-surface-tag",
		// "check-wait-before-scope-tag",
    };
    public static Method[] checker_methods = null;
    
    WindowController wc;

    boolean[] standard;
    boolean drop_from_log;

    public StrangeFilter() {
		this.wc = new WindowController(this);
		this.standard = new boolean[NUM_CHECK_ITEMS];
		this.drop_from_log = false;

		// 基準のデフォルト値を設定
		standard[CHK_NONTERMINATE] = true;
		standard[CHK_SCOPE] = true;
		standard[CHK_SURFACE] = true;
    }

    public StrangeFilter(Map memento) {
		this();
		standard[CHK_NONTERMINATE] =
			((Number)memento.get(for_memento_check_type[CHK_NONTERMINATE])).intValue() != 0;
		standard[CHK_SCOPE] =
			((Number)memento.get(for_memento_check_type[CHK_SCOPE])).intValue() != 0;
		standard[CHK_SURFACE] =
			((Number)memento.get(for_memento_check_type[CHK_SURFACE])).intValue() != 0;
		drop_from_log =
			((Number)memento.get("drop-also-from-log")).intValue() != 0;
    }

    public boolean getCheck(int type) {
		return standard[type];
    }

    protected void setCheck(int type, boolean flag) {
		standard[type] = flag;
    }

    public boolean getDropFromLog() {
		return drop_from_log;
    }

    protected void setDropFromLog(boolean bool) {
		drop_from_log = bool;
    }

    public void configure(Object target, Method ok, Method cancel) {
		wc.configure(target, ok, cancel);
    }

    public String getConfiguration() {
		StringBuffer conf = new StringBuffer();
		for (int i = 0; i < NUM_CHECK_ITEMS; i++) {
			if (standard[i]) {
				if (conf.length() > 0) {
					conf.append(", ");
				}
				conf.append(human_readable_check_type[i]);
			}
		}
		conf.insert(0, '[');
		conf.append(']');

		if (drop_from_log) {
			conf.append(" (drop log)");
		}

		return conf.toString();
    }

    public String getStatus() {
		return "ok";
    }

    public void destruct() {
    }

    public ActionReply doAction(ActionRequest req) {
		if (checker_methods == null) {
			// メソッドがキャッシュされてゐない。
			Pattern pat_minus = Pattern.compile("-");
			checker_methods = new Method[NUM_CHECK_ITEMS];
			for (int i = 0; i < NUM_CHECK_ITEMS; i++) {
				// for_memento_check_typeのハイフンをアンダースコアに換へたものが
				// チェック用のメソッド名である。
				try {
					checker_methods[i] = getClass().getMethod(
						pat_minus.matcher(for_memento_check_type[i]).replaceAll("_"),
						new Class[] {ActionRequest.class});
				}
				catch (Exception e) {
					System.out.println("Internal error of StrangeFilter.");
					e.printStackTrace();
				}
			}
		}

		boolean drop_required = false;
		for (int i = 0; i < NUM_CHECK_ITEMS; i++) {
			if (standard[i]) {
				try {
					// チェック用メソッドは、チェックに引掛つた場合にtrueを返す。
					drop_required =
						((Boolean)checker_methods[i].invoke(
							this, new Object[] {req})).booleanValue();
					if (drop_required) {
						break; // 一つでもチェックに引掛つたなら、これ以上チェックを續ける意味が無い。
					}
				}
				catch (Exception e) {
				}
			}
		}

		ActionReply rep = new ActionReply();
		if (drop_required) {
			rep.setDrop(drop_from_log);
		}
		return rep;
    }

    public Map getMemento() {
		// ログからの消去のフラグ及び全てのチェックフラグ
		Hashtable memento = new Hashtable();
		memento.put("drop-also-from-log", new Integer(drop_from_log ? 1 : 0));
		for (int i = 0; i < NUM_CHECK_ITEMS; i++) {
			memento.put(
				for_memento_check_type[i], new Integer(standard[i] ? 1 : 0));
		}
		return memento;
    }

    public static String getName() {
		return "Strange Filter";
    }

    
    // チェック用メソッド。doActionから間接的に呼ばれる。
    // 通過させないのであれば、trueを返す。
    
    public boolean check_nonterminate_tag(ActionRequest req) {
		return !req.getScript().startsWith("\u00a5t");
    }

    private static String regex_scope = "\u00a5[01hu]";
    protected static Pattern pat_scope = Pattern.compile(regex_scope);
    public boolean check_scope_tag(ActionRequest req) {
		return !pat_scope.matcher(req.getScript()).find();
    }

    // 好い加減な正規表現ですが、問題は無いでせう…。
    private static Pattern pat_surface = Pattern.compile("\u00a5s");
    public boolean check_surface_tag(ActionRequest req) {
		return !pat_surface.matcher(req.getScript()).find();
    }

/*    
	  private static Pattern pat_synch_and_url = Pattern.compile(
	  "\u00a5_s.+?\u00a5_s|\u00a5URL(?:\\[.+?\\])+");
	  private static Pattern pat_wait_before_scope = Pattern.compile(
	  regex_scope+".*?\u00a5_?w.*?"+regex_scope);
	  private static Pattern pat_nowait_before_scope = Pattern.compile(
	  regex_scope+".*?"+regex_scope);
	  public boolean check_wait_before_scope_tag(ActionRequest req) {*/
	/*
	  次のやうなロジックに従ふ。手抜きであり、問題となれば後に修正する。
	  1. \_sで圍まれた範圍は、\w9として扱ふ。
	  2. \URL[][]...も\w9として扱ふ。
	  3. スコープ切替〜\w〜スコープ切替という部分があればチェックを通過。
	  4. スコープ切替〜\_w〜スコープ切替という部分があつてもチェックを通過。
	  5. スコープ切替〜スコープ切替という部分があれば、チェックされる。
	  6. 最終的にはチェックを通過。
	*/
	/*String script =
	  pat_synch_and_url.matcher(req.getScript()).replaceAll("\u0015w9");
	
	  if (pat_wait_before_scope.matcher(script).find()) {
	  // 通過
	  return false;
	  }

	  if (pat_nowait_before_scope.matcher(script).find()) {
	  // チェック
	  return true;
	  }

	  return false;
	  }
	*/
}