/*
  $Id: Foundation.java,v 1.3 2006/09/08 14:37:41 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package core;
import core.*;
import log.*;
import log.logviewer.*;
import gui.pref.*;
import gui.main.*;
import java.net.*;
import java.io.*;
import java.util.*;
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;

/*
  ### Notification一覧 ###
    
  名称				UserInfo	条件
  BCGhostSpecModified	無し		prefウインドウが閉じられた。IfGhost指定の項目リストが変化した可能性がある。
*/

public class Foundation
{
    protected static Foundation _sharedFoundation;
    
    ServerConnection slpp_connection;
    SSTPQueue sstp_queue;
    LogModifier log_modifier;
    Httpd httpd;
    
    // ボトルを受け取った時のサウンド
    boolean enabled_sound_bottle_received;
    NSSound sound_bottle_received;
    
    public Foundation() {
        _sharedFoundation = this;
	
		// サウンドを読み込む
		String soundname;
		NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
		soundname = (String)defaults.objectForKey("sound_bottle_receive name");
		if (soundname != null) setSoundOfReceivingBottle(soundname);
		enabled_sound_bottle_received = (defaults.integerForKey("sound_bottle_receive enabled") == 1);
	
		// 古いログを圧縮
		LogFileCompressor.compressOldFiles();

		if (defaults.integerForKey("http-log-browser.enabled") != 0) {
			// ログビューアのHTTPDを起動
			startHttpd();
		}
    }

    public void startHttpd() {
		if (httpd == null) {
			NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
	    
			httpd = new Httpd(defaults.integerForKey("http-log-browser.port"));
			httpd.start();
		}
    }

    public void stopHttpd() {
		if (httpd != null) {
			httpd.terminate();
			httpd = null;
		}
    }
    
    public static Foundation sharedFoundation() {
        return _sharedFoundation;
    }
    
    public void applicationDidFinishLaunching(NSNotification aNotification) {
		// メインウインドウを開く前に環境設定を讀む。
		PrefWindowController.sharedPrefWindowController();
	
        MainWindowController mainWindowC = MainWindowController.sharedInstance();
        mainWindowC.showWindow(null);
        
        // キューはローカルテストでも使うので常に起動させておく。
        sstp_queue = new SSTPQueue();
        sstp_queue.start();
		
		//LogData log = LogDataServer.sharedLogServer().get(2003,3,2);
		//log.beDirty(LogData.LOG_ALL);
		//log.beDirty(LogData.LOG_USER);
		//log.commit();
    }
    
    public boolean slpp_connected() {
        if (slpp_connection != null && slpp_connection.isAlive())
			{
				return true;
			}
        else
			{
				return false;
			}
    }
    
    public void slpp_connect() {
        if (slpp_connected()) return;
        
        slpp_connection = new ServerConnection();
        slpp_connection.start();
    }
    
    public void slpp_disconnect() {
        if (!slpp_connected()) return;
        
        slpp_connection.terminate();
		slpp_connection = null;
    }
    
    public ServerConnection getSlppConnection() {
        return slpp_connection;
    }
    
    public SSTPQueue getQueue() {
        return sstp_queue;
    }
    
    public LogModifier getLogModifier() {
		return log_modifier;
    }
    
    public Vector sendToLocalSstpd(Vector req) {
        // localhost:11000に接続し、リクエストreqを送信します。
        // レスポンスは行毎に分割され、Vectorで返されます。
        // タイムアウトしたり通信エラーが発生したらnullが返ります。
        Socket s = null;
        try {
			NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
			String sstpservername = (String)defaults.objectForKey("sstpserver.servername");
			if (sstpservername == null) sstpservername = "localhost";
			int sstpport = defaults.integerForKey("sstpserver.sstpport");
			if (sstpport == 0) sstpport = 11000;
			s = new Socket(sstpservername,sstpport);
			
			// \URLが使われる可能性のため、タイムアウトはしない。
            
            InputStream is = s.getInputStream();
            OutputStream os = s.getOutputStream();
            BufferedReader br = new BufferedReader(new InputStreamReader(is,"SJIS")); // SJISと仮定。本当はこれでは駄目。
            OutputStreamWriter osw = new OutputStreamWriter(os,"SJIS");
            
            int n_lines = req.size();
            for (int i = 0;i < n_lines;i++) {
                writeStrUsingWriter(osw,(String)req.elementAt(i)); writeCRLF(os);
            }
            writeCRLF(os);
            
            // 反応を取得する
            Vector resp = new Vector();
            String line = null;
            while (true) {
                line = br.readLine();
                
                if (line == null) break;
                resp.add(line);
            }
            
            s.close();
            return resp;
        }
        catch (Exception e) {
            try{s.close();}catch(Exception ex){}
            return null;
        }
    }
    
    private void writeStrUsingWriter(OutputStreamWriter osw,String param) {
        try{  osw.write(param,0,param.length()); osw.flush();  }catch(Exception e){}
    }
    
    private void writeStr(OutputStream os,String param) {
        try{  os.write(param.getBytes(),0,param.length());  }catch(Exception e){}
    }
    
    private void writeCRLF(OutputStream os) {
        byte[] crlf = new byte[2];
        crlf[0] = 13;
        crlf[1] = 10;
        try{  os.write(crlf,0,2);  }catch(Exception e){}
    }
    
    public void applicationWillTerminate(NSNotification aNotification) {
        // SLPPセッションが生きていたらterminateする。
		LogDataServer.sharedLogServer().commitAll();
        slpp_disconnect();
        
        sstp_queue.terminate();
    }
    
    public void setSoundOfReceivingBottle(String soundname) {
		sound_bottle_received = NSSound.soundNamed(soundname);
    }
    
    public void playSoundOfReceiveBottle() {
		if (enabled_sound_bottle_received && sound_bottle_received != null) {
			sound_bottle_received.play();
		}
    }
    
    public void setEnabledOfSoundOfReceiveBottle(boolean b) {
		enabled_sound_bottle_received = b;
    }
}
