/*
  $Id: SSTPQueue.java,v 1.3 2006/09/08 14:37:42 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package core;
import core.*;
import gui.main.*;
import com.apple.cocoa.foundation.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.util.regex.*;

public class SSTPQueue extends Thread {
    private int interval = 1000; // 一秒おきに送る。
    
    Vector queue;
    private volatile boolean signal_stop = false;
    private volatile boolean signal_sleep = false;
    
    public SSTPQueue() {
        queue = new Vector();
    }
    
    public void run() {
		int pool = NSAutoreleasePool.push();
	
        MainWindowController.sharedInstance().setBottlesInQueueCount(0);

		NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
		Pattern pat_ifghost = Pattern.compile("^IfGhost\\s*:\\s*(.*)$");

		int pool_local = Integer.MIN_VALUE; // 安全かだうかは分からないが…。
        while (!signal_stop) {
			if (pool_local != Integer.MIN_VALUE) {
				NSAutoreleasePool.pop(pool_local);
			}
			pool_local = NSAutoreleasePool.push();
	    
            // キューが空だったりスリープ中だったりしたら
            if (signal_sleep || queue.size() <= 0) {
                try {
					// 負荷軽減
                    sleep(10 * 1000);
                }
                catch (InterruptedException e) {}
                continue;
            }
            
            // 先にプッシュされたリクエストを先に送る。
            Vector req = (Vector)queue.elementAt(0);
            Vector resp = Foundation.sharedFoundation().sendToLocalSstpd(req);
            
            if (resp == null) {
				// 通信エラーなので諦める。
				//queue.remove(0);
				//MainWindowController.sharedInstance().setBottlesInQueueCount(queue.size());
		
				// 2003/02/16 - 諦めない。
                continue;
            }
            
			/*
			  204または200であれば、成功したのでデキューして良い。
			  503 Service Unavailableであれば、設定によっては代替ゴースト宛てに再送する。
			  409以外のエラーなら、諦めてデキューして良い。
			  409だった場合のみ、デキューしてはならない。
			*/
            String first_line = (resp.size() == 0 ? "" : (String)resp.elementAt(0));
			if (first_line.indexOf("503") != -1) {
				// 設定を取得。
				switch (defaults.integerForKey("sstp-error.503.select")) {
				case 0:
					// 諦める
					queue.removeElementAt(0);
					break;

				case 1:
					// 代替ゴーストへ送る。
					String ghostname = (String)defaults.objectForKey("sstp-error.503.substitution");
					if (ghostname == null) {
						ghostname = "-UNINSTALLED-";
					}
					// IfGhostの行を探して置換へる。無ければ追加。
					boolean replaced = false;
					boolean drop_this = false;
					for (int i = 0; i < req.size(); i++) {
						String line = (String)req.get(i);
						Matcher m;
						if ((m = pat_ifghost.matcher(line)).find()) {
							// これはIfGhost行である。
							if (m.group(1).equals(ghostname)) {
								// これから送らうとしてゐる代替ゴースト宛てのボトルであつた。
								drop_this = true;
							}
							else {
								req.set(i, "IfGhost: "+ghostname);
								replaced = true;
							}
							break;
						}
					}
					if (drop_this) {
						queue.removeElementAt(0);
					}
					else {
						if (!replaced) {
							req.add("IfGhost: "+ghostname);
						}
						continue;
					}
					break;

				default:
					queue.removeElementAt(0);
				}
			}
			else if (first_line.indexOf("409") == -1) {
				queue.removeElementAt(0);
            }
            MainWindowController.sharedInstance().setBottlesInQueueCount(queue.size());
            
            try {
                sleep(interval);
            }
            catch (InterruptedException e) {}
        }
		if (pool_local != Integer.MIN_VALUE) {
			NSAutoreleasePool.pop(pool_local);
		}
        
        MainWindowController.sharedInstance().setBottlesInQueueCount(-1);
		NSAutoreleasePool.pop(pool);
    }
    
    public void terminate() {
        if (!isAlive()) return;
        
        signal_stop = true;
        interrupt();
        while (isAlive()) {}
    }
    
    public void push(Vector request) {
        if (!isAlive()) return;
        
        queue.add(request);
        MainWindowController.sharedInstance().setBottlesInQueueCount(queue.size());
        interrupt(); // プッシュされたことをすぐに気づかせる。
    }
    
    public void setSleeping(boolean val) {
        signal_sleep = val;
        if (isAlive()) interrupt(); // 状態が変化したことをすぐに気づかせる。
    }
    
    public void clear() {
		queue.clear();
		MainWindowController.sharedInstance().setBottlesInQueueCount(0);
    }
}
