/*
  $Id: ScriptAnalyzer.java,v 1.3 2006/09/08 14:37:42 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
  このクラスは本来スレッドセーフでないので、その窓口たるanalyze(Object)はsynchronized宣言されている。
*/

package core;
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.util.*;

public class ScriptAnalyzer {
    static final String[] percent_tags = {
		"%month","%day","%hour","%minute","%second","%username","%selfname2","%selfname","%keroname",
		"%ms","%mz","%ml","%mc","%mh","%mt","%me","%mp","%m?","%dms"
    };
    static Vector url_args;
    
    static NSColor tagcolor,errorcolor,varcolor,hontaicolor,unyuucolor,synchronizedcolor;
    static {
		tagcolor          = NSColor.colorWithCalibratedRGB(0.0f,0.5f,0.0f,1.0f);
		errorcolor        = NSColor.colorWithCalibratedRGB(0.9f,0.0f,0.0f,1.0f);
        varcolor          = NSColor.colorWithCalibratedRGB(0.0f,0.0f,0.5f,1.0f);
		hontaicolor       = NSColor.colorWithCalibratedRGB(0.831f,0.392f,0.0f,1.0f);
		unyuucolor        = NSColor.colorWithCalibratedRGB(0.0f,0.470f,0.518f,1.0f);
		synchronizedcolor = NSColor.colorWithCalibratedRGB(0.8f,0.0f,0.667f,1.0f);
    }
    
    public static synchronized boolean analyze(Object source) {
        // source: NSText又はNSMutableAttributedString。
        // スクリプト又はパラメータにエラーがあればtrueを返す。
        if (!(source instanceof NSText) && !(source instanceof NSMutableAttributedString)) {
			return true;
		}
	
		// init
		lastColor = null;
		lastStartingPoint = 0;
        
        String data = (source instanceof NSText ? ((NSText)source).string() : ((NSAttributedString)source).stringReference().string());
        final int len = data.length();
        
        if (source instanceof NSText) {
			((NSView)source).setNeedsDisplay(false);
            ((NSText)source).setTextColorInRange(null,new NSRange(0,len));
        }
        else {
            ((NSMutableAttributedString)source).setAttributesInRange(null,new NSRange(0,len));
        }
        
        // バックスラッシュを￥に置き換える
        if (data.indexOf('\\') != -1) {
            StringBuffer sb = new StringBuffer(data);
            
            int pos;
            while ((pos = sb.toString().indexOf('\\')) != -1) {
                sb.setCharAt(pos,'\u00a5');
            }
            
            data = sb.toString();
            
            if (source instanceof NSText) {
				NSRange savedrange = ((NSText)source).selectedRange();
                ((NSText)source).setString(data);
				((NSText)source).setSelectedRange(savedrange);
            }
            else {
                ((NSMutableAttributedString)source).replaceCharactersInRange(new NSRange(0,data.length()),data);
            }
        }
	
		boolean hontaiContext = true; // 本体にフォーカスが当たっている時にtrueになります。
		boolean synchronizedContext = false; // シンクロナイズドセッション中。これがtrueの時はhontaiContextは無視されます。
        
        int pos = 0;
        boolean error = false;
        while (pos < len) {
            if (pos == len - 1) { // 最後の文字だった
                // 通常文字のみ許可
                if (data.charAt(pos) == '\u00a5' || data.charAt(pos) == '\\' || data.charAt(pos) == '%') {
                    // ERROR
                    setColor(source,errorcolor,pos,1);
                    error = true;
                    pos++;
                }
				else {
					if (synchronizedContext) {
						setColor(source,synchronizedcolor,pos,1);
					}
					else {
						setColor(source,(hontaiContext ? hontaicolor : unyuucolor),pos,1);
					}
				}
                break;
            }
            
            switch (data.charAt(pos)) {
			case	'\u00a5': // ￥
			case	'\\':
                if ("thuec".indexOf(data.charAt(pos+1)) != -1) { // これらはパラメータを取らない
                    // えんいーの後の文字は全てエラー。
                    if (data.charAt(pos+1) == 'e' && pos+1 != len-1) {
                        // 以上二文字をtagcolorにする。
                        setColor(source,tagcolor,pos,2);
                        // ERROR
                        setColor(source,errorcolor,pos+2,len - (pos+2));
                        error = true;
                        pos = len;
                    }
                    else {
						if (data.charAt(pos+1) == 'h') {
							hontaiContext = true;
						}
						else if (data.charAt(pos+1) == 'u') {
							hontaiContext = false;
						}
			
                        // 以上二文字をtagcolorにする。
                        // OK
                        setColor(source,tagcolor,pos,2);
                        pos += 2;
                    }
                }
                else if (data.charAt(pos+1) == 'w') { // これは数字一文字を取る
                    if (pos + 1 == len - 1) { // \wで終わってた
						// ERROR
                        setColor(source,errorcolor,pos,2);
                        error = true;
                        pos += 2;
                    }
                    else {
                        if (Character.isDigit(data.charAt(pos+2))) {
                            // OK
                            setColor(source,tagcolor,pos,3);
                            pos += 3;
                        }
                        else {
                            // ERROR
                            setColor(source,errorcolor,pos,3);
                            error = true;
                            pos += 3;
                        }
                    }
                }
				else if (data.charAt(pos+1) == '_') { // \_s及び\_qのみ許可。
					if (pos + 1 == len - 1) { // \_で終わってた
						// ERROR
                        setColor(source,errorcolor,pos,2);
                        error = true;
                        pos += 2;
                    }
					else if (data.charAt(pos+2) == 's') { // \_sタグ。許可。
						synchronizedContext = !synchronizedContext;
						// OK
						setColor(source,tagcolor,pos,3);
						pos += 3;
					}
					else if (data.charAt(pos+2) == 'q') { // \_qタグ。許可。
						// OK
						setColor(source,tagcolor,pos,3);
						pos += 3;
					}
					else { // それ以外の\_タグは不許可
						// ERROR
						setColor(source,errorcolor,pos,3);
						error = true;
						pos += 3;
					}
				}
				else if (data.charAt(pos+1) == 'U') { // \URLのみ許可。一番複雑。
					if (pos + 1 == len - 1) { // \Uで終わってた
						// ERROR
						setColor(source,errorcolor,pos,2);
                        error = true;
                        pos += 2;
                    }
					else if (data.startsWith("\u00a5URL[",pos)) { // \URL[まで確認
						// [...][...]の組み合わせを配列に入れていく。
						if (url_args == null) {
							url_args = new Vector();
						}
						else {
							url_args.clear();
						}
						int blockstart = pos+4;
						int blockend;
						while (true) {
							blockend = blockstart; // この一文字後から検索
							while (true) {
								blockend = data.indexOf(']',blockend+1);
								if (blockend == -1) break;
								if (data.charAt(blockend-1) != '\u00a5') {
									break; // エスケープされていたらもう一度。
								}
							}
							if (blockend == -1) { // つまり見つからなかった
								break; // 終了
							}
							else {
								url_args.add(data.substring(blockstart+1,blockend));
								blockstart = blockend + 1;
								if (len <= blockstart || data.charAt(blockstart) != '[') {
									break; // 直後から再び始まっていなければ終了。
								}
							}
						}
						if (blockend == -1) { // 括弧が閉じられていなかった場合。
							setColor(source,errorcolor,pos,len - pos); // 残りのスクリプト全体。
							error = true;
							pos = len;
						}
						else if (url_args.size() <= 0 || url_args.size() % 2 == 0) { // 偶数ではいけない。
							// ERROR
							setColor(source,errorcolor,pos,blockend - pos + 1); // 最後の閉じ括弧まで。
							error = true;
							pos = blockend+1;
						}
						else if (url_args.size() == 1) { // 一つしか無かった場合はそれをURLチェック。
							String arg = (String)url_args.elementAt(0);
							if (arg.startsWith("http://")) {
								// OK
								setColor(source,tagcolor,pos,blockend - pos + 1); // 最後の閉じ括弧まで。
								pos = blockend+1;
							}
							else {
								// ERROR
								setColor(source,errorcolor,pos,blockend - pos + 1); // 最後の閉じ括弧まで。
								error = true;
								pos = blockend+1;
							}
						}
						else { // 複数あれば偶数番目のみURLチェック。
							int n_args = url_args.size();
							boolean good = true;
							for (int i = 2;i <= n_args;i += 2) {
								String arg = (String)url_args.elementAt(i-1);
								if (!arg.startsWith("http://")) {
									// BAD
									good = false;
								}
							}
							if (good) {
								// OK
								setColor(source,tagcolor,pos,blockend - pos + 1); // 最後の閉じ括弧まで。
								pos = blockend+1;
							}
							else {
								// ERROR
								setColor(source,errorcolor,pos,blockend - pos + 1); // 最後の閉じ括弧まで。
								error = true;
								pos = blockend+1;
							}
						}
					}
					else {
						// ERROR
                        setColor(source,errorcolor,pos,2);
                        error = true;
                        pos += 2;
					}
				}
				else if (data.charAt(pos+1) == 'n') { // これは[]に囲まれたパラメータを一つ取るかも知れない。
					if (data.charAt(pos+2) == '[') {
						// ]を探す
						int blockend = data.indexOf(']', pos+3);
						if (blockend == -1) {
							// ERROR
							setColor(source,errorcolor,pos,len - pos);
							error = true;
							pos = len;
						}
						else {
							String param = data.substring(pos+3, blockend);
							// 現在許されているパラメータは"half"のみ。
							if (param.equals("half")) {
								// OK
								setColor(source, tagcolor, pos, blockend - pos + 1);
								pos = blockend + 1;
							}
							else {
								// ERROR
								setColor(source,errorcolor,pos,blockend - pos + 1);
								error = true;
								pos = blockend + 1;
							}
						}
					}
					else {
						// OK
						setColor(source, tagcolor, pos, 2);
						pos += 2;
					}
				}
                else if (data.charAt(pos+1) == 's') { // これは数字一文字又は[]に囲まれた複数の文字のパラメータを取る
                    if (pos + 1 == len - 1) { // \sで終わってた
						// ERROR
                        setColor(source,errorcolor,pos,2);
                        error = true;
                        pos += 2;
                    }
                    else {
						char thirdchar = data.charAt(pos+2);
                        if (Character.isDigit(thirdchar)) {
                            // OK
							setColor(source,tagcolor,pos,3);
							pos += 3;
                        }
                        else if (thirdchar == '[') {
                            // 閉じ括弧が来るまでの文字は全て数字でなければならない。
                            int blockend = data.indexOf(']',pos+2);
                            if (blockend == -1) {
								// ERROR
                                setColor(source,errorcolor,pos,len - pos);
                                error = true;
                                pos = len;
                            }
                            else if (blockend == pos+2) { // []になってた。
                                // ERROR
                                setColor(source,errorcolor,pos,4);
                                error = true;
                                pos += 4;
                            }
                            else {
                                try {
                                    int param = Integer.parseInt( data.substring(pos+3,blockend) );
									if (param >= 0 || param == -1) {
										// OK
                                        setColor(source,tagcolor,pos,blockend - pos + 1);
                                        pos = blockend + 1;
                                    }
                                    else {
                                        // ERROR
                                        setColor(source,errorcolor,pos,blockend - pos + 1);
                                    	error = true;
                                        pos = blockend + 1;
                                    }
                                }
                                catch (Exception e) {
                                    // ERROR
                                    setColor(source,errorcolor,pos,blockend - pos + 1);
                                    error = true;
                                    pos = blockend + 1;
                                }
                            }
                        }
                        else { // 数字でも[でも￥でも%でもなかったら
                            // ERROR
                            setColor(source,errorcolor,pos,3);
                            error = true;
                            pos += 3;
                        }
                    }
                }
                else if (data.charAt(pos+1) == '\u00a5' || data.charAt(pos+1) == '\\' || data.charAt(pos+1) == '%') { // エスケープ
                    // OK
					setColor(source,(hontaiContext ? hontaicolor : unyuucolor),pos,2);
                    pos += 2;
                }
                else { // 未定義又は禁止￥タグ
                    // ERROR
                    setColor(source,errorcolor,pos,2);
                    error = true;
                    pos += 2;
                }
                break;
                
			case	'%':
                boolean found = false;
                for (int i = 0;i < percent_tags.length;i++) {
                    if (data.startsWith(percent_tags[i],pos)) {
						// OK
                        found = true;
                        setColor(source,varcolor,pos,percent_tags[i].length());
                        pos += percent_tags[i].length();
                        break;
                    }
                }
                if (!found) {
                    /*
					// ERROR
                    setColor(source,errorcolor,pos,1);
                    error = true;
                    pos++;
					*/
					if (synchronizedContext) {
						setColor(source,synchronizedcolor,pos,1);
					}
					else {
						setColor(source,(hontaiContext ? hontaicolor : unyuucolor),pos,1);
					}
					pos++;
                }
                break;
                
			default:
				if (synchronizedContext) {
					setColor(source,synchronizedcolor,pos,1);
				}
				else {
					setColor(source,(hontaiContext ? hontaicolor : unyuucolor),pos,1);
				}
                pos++;
            }
        }
	
		// clean-up
		setColor(source,null,len,0);
	
		if (source instanceof NSText) {
			((NSView)source).setNeedsDisplay(true);
        }
	
        return error;
    }
    
    private static NSColor lastColor = null;
    private static int lastStartingPoint = 0;
    private static void setColor(Object source,NSColor color,int pos,int len) {
		// color == nullの場合はクリーンアップを行います。
		// その際posには必ず長さを指定して下さい。
		
		// このメソッドは先頭からシーケンシャルに使用されなければなりません。
		// 任意の場所に色を指定することは出来ません。
		
		// 使用前に必ずlastColorとlastStartingPointを初期化して下さい。
		
		if (color == lastColor) {
			return;
		}
		else {
			// まずはlastStartingPointから範囲の直前までをlastColorにセット
			if (lastColor != null) {
				try {
					NSRange range = new NSRange(lastStartingPoint,(pos-1) - lastStartingPoint + 1);
					if (source instanceof NSText) {
						((NSText)source).setTextColorInRange(lastColor,range);
					}
					else {
						((NSMutableAttributedString)source).addAttributeInRange(NSAttributedString.ForegroundColorAttributeName,lastColor,range);
					}
				}
				catch (Exception e) {}
			}
			
			// 各種変数をセット
			lastColor = color;
			lastStartingPoint = pos;
		}
	}
    
    public static String expandYenURL(String script) {
		// \URLタグを展開します。
		// 文法エラーが無いことを前提に処理します。不安なら予めanalyze()でチェックしておいて下さい。
		int n_enq_exists = 0;
		
		StringBuffer buf = new StringBuffer(script);
		int pos;
		while ((pos = buf.toString().indexOf('\\')) != -1) {
			buf.setCharAt(pos,'\u00a5');
		}
	
		pos = 0;
		while (pos < buf.length()) {
			pos = buf.toString().indexOf("\u00a5URL",pos);
			if (pos == -1) break; // end
			if (pos >= 1 && buf.charAt(pos-1) == '\u00a5') {pos++; continue;} // escaped (本当は\\を考えるべきだが‥‥)
			
			Vector args = new Vector();
			int blockstart = pos+4;
			int blockend;
			while (true) {
				blockend = blockstart; // この一文字後から検索
				while (true) {
					blockend = buf.toString().indexOf(']',blockend+1);
					if (buf.charAt(blockend-1) != '\u00a5') break; // エスケープされていたらもう一度。
				}
				args.add(buf.substring(blockstart+1,blockend));
				blockstart = blockend + 1;
				if (buf.length() <= blockstart || buf.charAt(blockstart) != '[') break; // 直後から再び始まっていなければ終了。
			}
			if (args.size() == 1) { // YES/NO style
				buf.replace(
					pos,blockend+1,
					"\u00a5q"+n_enq_exists+'['+(String)args.elementAt(0)+"][Go]" +
					"\u00a5q"+(n_enq_exists+1)+"[][Cancel]"
					);
				n_enq_exists += 2;
			}
			else { // TABLE style
				// キャンセルは最後に持っていく。
				StringBuffer enqtags = new StringBuffer();
				final int n_args = args.size();
				for (int i = 1;i < n_args;i += 2) {
					String url = (String)args.elementAt(i);
					String label = (String)args.elementAt(i+1);
					
					enqtags.append("\u00a5q"+n_enq_exists+'['+url+"]["+label+']');
					n_enq_exists++;
				}
				enqtags.append("\u00a5q"+n_enq_exists+"[]["+(String)args.elementAt(0)+']');
				n_enq_exists++; // 完成
				
				buf.replace(pos,blockend+1,enqtags.toString());
			}
		}
		return buf.toString();
	}
}
