/*
  $Id: ServerConnection.java,v 1.4 2006/09/08 14:37:42 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package core;
import core.*;
import log.*;
import gui.*;
import gui.main.*;
import action.*;
import com.apple.cocoa.foundation.*;
import java.net.*;
import java.io.*;
import java.util.*;

public class ServerConnection extends Thread {
    protected static final boolean DEBUG = false;
    protected static void debugstr(String str) {
		if (DEBUG) System.out.println(str);
    }
    
    volatile boolean signal_stop = false;
    
    MainWindowController mainWindow;
    
    Socket s;
    
    String luid;
    
    Vector chList;

    String slpp_host;
    int slpp_port;
    
    public ServerConnection() {
        chList = new Vector();
        mainWindow = MainWindowController.sharedInstance();
    }

    public String getSlppHost() {
		return slpp_host;
    }

    public int getSlppPort() {
		return slpp_port;
    }
    
    public void run() {
		int pool = NSAutoreleasePool.push();
		
		try {
			_run();
		} catch (Exception e) {
			e.printStackTrace();
		}
		closeConnection();
		
		NSAutoreleasePool.pop(pool);
	}
	protected void _run() throws Exception {
		// この関数からreturnなどで抜ければ正常切断、例外で抜ければ異常切断する。
		
		luid = getLUID(false);
		getChannels(); // チャンネルリスト取得
		
		String app_name = NSBundle.mainBundle().localizedStringForKey("CFBundleShortVersionString","BotCli","InfoPlist");
        
		boolean use_proxy = false;
		String bottleserver_host;
		int bottleserver_slppport;
		try {
			NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
			bottleserver_host = (String)defaults.objectForKey("bottleserver.servername");
			if (bottleserver_host == null) bottleserver_host = "bottle.mikage.to";
			bottleserver_slppport = defaults.integerForKey("bottleserver.slppport");
			if (bottleserver_slppport == 0) bottleserver_slppport = 9871;

			slpp_host = bottleserver_host;
			slpp_port = bottleserver_slppport;
			
			use_proxy = defaults.integerForKey("proxy.http.use") == 1;
			String http_proxy_host = (String)defaults.objectForKey("proxy.http.host");
			if (http_proxy_host == null || http_proxy_host.length() == 0) use_proxy = false;
			int http_proxy_port = defaults.integerForKey("proxy.http.port");
			if (defaults.objectForKey("proxy.http.port") == null) use_proxy = false;
			
			if (use_proxy) {
				s = new Socket(http_proxy_host,http_proxy_port);
			}
			else {
				s = new Socket(bottleserver_host,bottleserver_slppport);
			}
		}
		catch (Exception e) {
			// エラー処理。そのうち。
			System.out.println("Connection failed.");
			e.printStackTrace();
			return;
		}
        
		// 接続できたので、Connectedインジケータをonにする。
		mainWindow.setConnectedState(true);
		mainWindow.setChannelRegisteredCount(0);
		
		// LUIDを送り、SLPP2.0セッション開始。
		mainWindow.setSendState(true);
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(s.getOutputStream(),"SJIS"));
		if (use_proxy) {
			bw.write("POST http://"+bottleserver_host+":"+bottleserver_slppport+"/ HTTP/1.0\r\n");
		}
		else {
			bw.write("POST / HTTP/1.0\r\n");
		}
		bw.write("User-Agent: "+app_name+"\r\n");
		bw.write("Content-Length: "+luid.length()+"\r\n");
		bw.write("Content-Type: text/plain; charset=Shift_JIS\r\n");
		bw.write("\r\n");
		bw.write(luid);
		bw.write("\r\n");
		bw.flush();
		mainWindow.setSendState(false);
		
		BufferedReader br = new BufferedReader(new InputStreamReader(s.getInputStream(),"SJIS")); // 暫定処理
		String line;
		
		// SLPP鯖からのレスポンス。先頭のレスポンス以外は無視する。
		// 空の行までを読み呼ばす。
		s.setSoTimeout(30 * 1000);
		
		mainWindow.setReceiveState(true);
		line = br.readLine();
		if (line == null) throw new Exception();
		if (line.indexOf("200 OK") == -1) {
			System.out.println("SLPP Server returned: "+line);
			return; // 200 OKが含まれていなかったら切断。無念。
		}
		
		while (true) {
			try {
				line = br.readLine();
				if (line == null) throw new Exception("");
				if (line.trim().length() == 0) break;
			}
			catch (InterruptedIOException e) {
				if (signal_stop) return;
			}
		}
		mainWindow.setReceiveState(false);
		
		// 以下、切断まで永久に読み込みループ。
		String command;
		Vector params = new Vector();
		while (!signal_stop) {
			s.setSoTimeout(1 * 1000); // 普段は1秒毎にsignal_stopを見る。
			try {
				line = br.readLine();
				if (line == null) {
					throw new Exception(""); // サーバーが落ちでもしない限り起こらないだろうが。
				}
				else {
					line = line.trim();
				}
				if (line.length() == 0) continue;
				
				// lineに中身があった。
				
				mainWindow.setReceiveState(true);
				s.setSoTimeout(30 * 1000); // データは少ない量でまとまって来るので、タイムアウトはなるべくしないようにしておく。
				
				// 空の行が来るまで読み込みを続ける
				// SLPP1.1は追加データの類いを返さないようなので、そのデータはそのままSSTPリクエストとして処理してよい。
				command = line;
				while (true) {
					line = br.readLine();
					if (line == null) {
						throw new Exception("");
					}
					else {
						line = line.trim();
					}
					if (line.length() == 0) break; // 空の行が来たらこのリクエスト終了。
					
					params.add(line);
				}
				doCommand(command,params);
				params.clear();
				
				mainWindow.setReceiveState(false);
			}
			catch (InterruptedIOException e) {
				// タイムアウトした。
				// 別に処理は必要ない。
			}
		}
	}
    
    private void closeConnection() {
		// 終了処理
		try {
			s.close();
		}
		catch (Exception e){}
		
		mainWindow.setConnectedState(false);
		mainWindow.setReceiveState(false);
		mainWindow.setSendState(false);
		mainWindow.setUsersCount(-1);
		mainWindow.setChannelsCount(-1);
		mainWindow.setChannelRegisteredCount(-1);
		
		// SLPPから切断してしまえばもうチャンネルリストは更新されないので無効である。
		// よって、リストを消去する。
		chList.clear();
		mainWindow.reloadChListView();
	}
    
    private Hashtable slppSession(String param) {
		// SLPP2.0セッションを実行します。
		// bottle.mikage.to:80/bottle2.cgiと交信します。
		// なお、Agentヘッダはここで付けますので、あらかじめ付けておく必要はありません。
		Socket s = null;
		Hashtable resp = null;
        
		String app_name = NSBundle.mainBundle().localizedStringForKey("CFBundleShortVersionString","BotCli","InfoPlist");
        
		try {
			param = "Agent: "+app_name+"\r\n"+param;
			
			NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
			String botservername = (String)defaults.objectForKey("bottleserver.servername");
			if (botservername == null) botservername = "bottle.mikage.to";
			int httpport = defaults.integerForKey("bottleserver.httpport");
			if (httpport == 0) httpport = 80;
						
			boolean use_proxy = defaults.integerForKey("proxy.http.use") == 1;
			String http_proxy_host = (String)defaults.objectForKey("proxy.http.host");
			if (http_proxy_host == null || http_proxy_host.length() == 0) use_proxy = false;
			int http_proxy_port = defaults.integerForKey("proxy.http.port");
			if (defaults.objectForKey("proxy.http.port") == null) use_proxy = false;
			
			if (use_proxy) {
				s = new Socket(http_proxy_host,http_proxy_port);
			}
			else {
				s = new Socket(botservername,httpport);
			}
			resp = slppSession(s,use_proxy,botservername,httpport,app_name,param);
		}
		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		finally {
			try{
				s.close();
			}
			catch(Exception e) {}
			
			mainWindow.setReceiveState(false);
		}
	
		// Result:フィールドを読んで、エラーだったらメッセージ表示。
		if (((String)resp.get("Result")).equals("Err")) {
			String message = (String)resp.get("ExtraMessage");
			if (message != null) {
				ServerErrorWindowController errorwin = new ServerErrorWindowController(message);
				errorwin.show();
			}
		}
        
		return resp;
	}
	
	protected Hashtable slppSession(Socket s,boolean use_proxy,String bottleserver_host,int bottleserver_httpport,String user_agent,String param) throws Exception {
		// この関数から戻ればsは自動的にcloseされ、Receiveインジケータは消灯する。
		// bottleserver_hostがnullでなければ、sはプロクシへの接続と見做す。
		Hashtable resp = new Hashtable();
		s.setSoTimeout(40 * 1000);
		
		mainWindow.setSendState(true);
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(s.getOutputStream(),"SJIS"));
		if (use_proxy) {
			bw.write("POST http://"+bottleserver_host+":"+bottleserver_httpport+"/bottle2.cgi HTTP/1.0\r\n");
		}
		else {
			bw.write("POST /bottle2.cgi HTTP/1.0\r\n");
		}
		bw.write("Host: "+bottleserver_host+":"+bottleserver_httpport+"\r\n");
		bw.write("User-Agent: "+user_agent+"\r\n");
		bw.write("Content-Type: text/plain; charset=Shift_JIS\r\n");
		bw.write("Content-Length: "+param.getBytes("SJIS").length+"\r\n");
		bw.write("\r\n");
		bw.write(param);
		bw.write("\r\n");
		bw.flush();
		mainWindow.setSendState(false);
            
		BufferedReader br = new BufferedReader(new InputStreamReader(s.getInputStream(),"SJIS")); // 暫定処理
		String line;
            
		mainWindow.setReceiveState(true);
		// まずはサーバーレスポンスを見る
		line = br.readLine();
		if (line == null || line.indexOf("200 OK") == -1) { // レスポンスのどこにも200 OKが入っていなかったら
			throw new Exception("Server returned "+line);
		}
            
		// 空行が来るまで読み飛ばす
		while (true) {
			line = br.readLine();			
			if (line == null) {
				throw new Exception();
			}
			else {
				line = line.trim();
			}
			if (line.length() == 0) break; // ここで読み飛ばし終了
		}
            
		// データを受信
		while (true) {
			line = br.readLine();
			if (line == null) {
				break; // 終了
			}
			else {
				line = line.trim();
			}
			if (line.length() == 0) break; // 終了
			// コロンで分割
			int colon_pos = line.indexOf(':');
			if (colon_pos == -1) throw new Exception("Server returned an invalid line: "+line);
			
			String type = line.substring(0,colon_pos).trim();
			String data = line.substring(colon_pos + 1).trim();
			resp.put(type,data);
		}
		return resp;
	}
    
    public String getLUID(boolean force_new) {
		// force_newがfalseの場合、defaultsからLUIDを読み出して、もしdefaultsに無かったら新たに取得して返します。
		// force_newがtrueの場合、defaulsとは無関係に新たに取得して返します。
		// defaultsに無くて取得にも失敗した場合はnullを返します。
		String luid;
		NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
        
		if (!force_new) {
			luid = (String)defaults.objectForKey("luid");
			if (luid != null && luid.length() == 82) { // 正しいLUIDが入っていた
				return luid;
			}
		}
        
		Hashtable resp = slppSession("Command: getNewId");
		String result = (String)resp.get("Result");
		if (result == null && !result.equals("OK")) {
			// 取得失敗。無念。
			return null;
		}
        
		// defaultsに保存
		String newid = (String)resp.get("NewID");
		defaults.setObjectForKey(newid,"luid");
		defaults.synchronize();
        
		return newid;
	}
    
    public void getChannels() {
		// getChannelsを送信して、チャンネルリストを更新します。
		chList.clear();
        
		Hashtable resp = slppSession("Command: getChannels");
        
		String result = (String)resp.get("Result");
		if (result == null && !result.equals("OK")) {
			// 取得失敗。無念。
			mainWindow.reloadChListView();
			return;
		}
        
		int n_ch = 0;
		try{ n_ch = Integer.parseInt( (String)resp.get("Count") ); }catch(Exception e){}
		mainWindow.setChannelsCount(n_ch);
        
		for (int i = 1;i <= n_ch;i++) {
			String name = (String)resp.get("CH"+i+"_name");
			String info = (String)resp.get("CH"+i+"_info");
			int count = 0;
			try{  count = Integer.parseInt( (String)resp.get("CH"+i+"_count") );  }catch(Exception e){}
			String ghost = (String)resp.get("CH"+i+"_ghost");
			int np = 0;
			try{  np = Integer.parseInt( (String)resp.get("CH"+i+"_nopost") );  }catch(Exception e){}
			boolean nopost = (np == 1 ? true : false);
            
			chList.add(new ChannelInfo(name,info,count,ghost,nopost));
		}
        
		mainWindow.reloadChListView();
	}
    
    public void loginChannels(Vector channels) {
		// 現在登録しているチャンネルのリストにchannelsを追加した後、setChannelsを呼び出します。
		if (channels.size() == 0) return;
		Vector new_array = new Vector(channels);
        
		int n_ch = chList.size();
		for (int i = 0;i < n_ch;i++) {
			ChannelInfo info = (ChannelInfo)chList.elementAt(i);
			if (info.registered())
				{
					// new_array内を検索して、既にエントリされていたら無視する。
					boolean found = false;
					int n_newarray = new_array.size();
					for (int j = 0;j < n_newarray;j++) {
						if (((String)new_array.elementAt(j)).equals(info.name())) {
							found = true;
							break;
						}
					}
					if (!found) {
						new_array.add(info.name());
					}
				}
		}
        
		setChannels(new_array);
	}
    
    public void logoutChannels(Vector channels) {
		// 現在登録しているチャンネルのリストからchannelsを消した後、setChannelsを呼び出します。
		if (channels.size() == 0) return;
		Vector new_array = new Vector();
        
		int n_ch = chList.size();
		for (int i = 0;i< n_ch;i++) {
			ChannelInfo info = (ChannelInfo)chList.elementAt(i);
			if (!info.registered()) continue; // 登録済みでなければ用はない。
            
			int n_ch_to_logout = channels.size();
			boolean found = false;
			for (int j = 0;j < n_ch_to_logout;j++) {
				if (info.name().equals( (String)channels.elementAt(j) )) {
					found = true;
					break;
				}
			}
			if (found) {
				info.setRegistered(false);
			}
			else {
				new_array.add(info.name());
			}
		}
        
		setChannels(new_array);
	}
    
    public void setChannels(Vector channels) {
		// SLPP2.0CGIにsetChannelsを送ります。
		// channelsはチャンネル名の配列です。
		if (!isAlive()) return;
        
		StringBuffer command = new StringBuffer();
		command.append("Command: setChannels\r\n");
		command.append("LUID: "+luid+"\r\n");
        
		int n_ch_to_register = channels.size();
		for (int i = 0;i < n_ch_to_register;i++) {
			try {
				command.append("Ch"+(i+1)+": "+(String)channels.elementAt(i)+"\r\n");
			}
			catch (Exception e) {}
		}
        
		// command完成。
        
		Hashtable resp = slppSession(command.toString());
		// 「Result: OK」ならチャンネルリストのregisteredフィールドを更新する。
		if (((String)resp.get("Result")).equals("OK")) {
			int n_ch = chList.size();
			for (int i = 0;i < n_ch;i++) {
				ChannelInfo info = (ChannelInfo)chList.elementAt(i);
				String itsName = info.name();
                
				// itsNameがchannelsの中に含まれているかどうかを調べる。
				boolean found = false;
				for (int j = 0;j < n_ch_to_register;j++) {
					if ( itsName.equals( (String)channels.elementAt(j) ) ) {
						found = true;
						break;
					}
				}
				info.setRegistered(found);
			}
			mainWindow.reloadChListView();
		}
	}
    
    public void doCommand(String command,Vector params) {
		/*System.out.println(command);
		  int n_params = params.count();
		  for (int i = 0;i < n_params;i++)
		  {
		  System.out.println( (String)params.objectAtIndex(i) );
		  }
		  System.out.println("");*/
        
		// SLPP2.0サーバーから送られてくるメッセージを解釈して実行する。
		if (command.equals("allUsers")) {
			docommand_allUsers(params);
		}
		else if (command.equals("channelUsers")) {
			docommand_channelUsers(params);
		}
		else if (command.equals("channelList")) {
			docommand_channelList(params);
		}
		else if (command.equals("broadcastMessage")) {
			docommand_broadcastMessage(params);
		}
		else if (command.equals("unicastMessage")) {
			docommand_unicastMessage(params);
		}
		else if (command.equals("forceBroadcastMessage")) {
			docommand_forceBroadcastMessage(params);
		}
		else if (command.equals("forceBroadcastInformation")) {
			docommand_forceBroadcastInformation(params);
		}
		else if (command.equals("closeChannel")) {
			docommand_closeChannel(params);
		}
	}
    
    public void docommand_allUsers(Vector params) {
		// パラメータは、「Num: [n_users]」の一行のみ。
		int n_users = 0;
		String param = (String)params.elementAt(0);
		try{  n_users = Integer.parseInt( param.substring(4).trim() );  }catch(Exception e){}
        
		mainWindow.setUsersCount(n_users);
	}
    
    public void docommand_channelUsers(Vector params) {
		String name = null;
		int newcount = 0;
        
		int n_params = params.size();
		for (int i = 0;i < n_params;i++) {
			String line = (String)params.elementAt(i);
            
			if (line.startsWith("Channel:")) {
				name = line.substring(8).trim();
			}
			else if (line.startsWith("Num:")) {
				try{ newcount = Integer.parseInt(line.substring(4).trim()); }catch(Exception e){}
			}
		}
        
		ChannelInfo info = findChInfoNamed(name);
		if (info != null) {
			info.setCount(newcount);
			mainWindow.reloadChListView();
		}
	}
    
    public void docommand_channelList(Vector params) {
		// とりあえずはNum:を画面に表示して、Entry:の最後についてくるカウント情報を更新するだけでいいか。
		int n_params = params.size();
		for (int i = 0;i < n_params;i++) {
			String line = (String)params.elementAt(i);
            
			if (line.startsWith("Entry:")) {
				String name = line.substring(6, line.lastIndexOf(' ') ).trim();
				String countdata = line.substring( line.lastIndexOf('(')+1 , line.lastIndexOf(')') ).trim();
				int newcount = 0;
				try{ newcount = Integer.parseInt(countdata); }catch(Exception e){}
                
				ChannelInfo info = findChInfoNamed(name);
				if (info != null) {
					info.setCount(newcount);
				}
			}
			else if (line.startsWith("Num:")) {
				int n_regd_ch = 0;
				try{ n_regd_ch = Integer.parseInt(line.substring(4).trim()); }catch(Exception e){}
                
				mainWindow.setChannelRegisteredCount(n_regd_ch);
			}
		}
		mainWindow.reloadChListView();
	}
    
    public void docommand_broadcastMessage(Vector param) {
        String mid = null,channel = null,uid = null,script = null,ifghost = null;
        
        int n_params = param.size();
        for (int i = 0;i < n_params;i++) {
            String line = (String)param.elementAt(i);
            if (line.startsWith("MID:")) {
                mid = line.substring(4).trim();
            }
            else if (line.startsWith("Channel:")) {
                channel = line.substring(8).trim();
            }
            else if (line.startsWith("SenderUID:")) {
                uid = line.substring(10).trim();
            }
			else if (line.startsWith("Script:")) {
				script = line.substring(7).trim();
			}
			else if (line.startsWith("IfGhost:")) {
				ifghost = line.substring(8).trim();
			}
		}
	
		String ghost_destination,ghost_default;
		// 指定されたチャンネルを探し、ゴースト名を取得。
		ChannelInfo ci = findChInfoNamed(channel);
		if (ci == null)  {
			// fatal error
			System.err.println("ServerConnection : fatal error. couldn't get ghost name of channel "+channel);
		}
		ghost_default = ci.ghost();
		ghost_destination = (ifghost == null ? ghost_default : ifghost); // 明示的にIfGhostが指定されていたらそれを潰さない。
	
		// /tmp/botcli-received-bottleがあれば書き込む。
		File fifo = new File("/tmp/botcli-received-bottle");
		if (fifo.exists()) {
			try {
				BufferedWriter bw = new BufferedWriter(
					new OutputStreamWriter(
						new FileOutputStream(fifo),"UTF-8"));
				bw.write("Charset: UTF-8\n");
				bw.write("IfGhost: "+ghost_destination+"\n");
				bw.write("Script: "+script+"\n");
				bw.close();
			}
			catch (Exception e) {}
		}

		// アクションに通す
		ActionReply act = ActionManager.sharedManager().doAction(
			false, channel, ifghost==null?"":ifghost, script, slpp_host+":"+slpp_port);
	
		if (act.getDrop() == 0) {
			// BotCli:new:MM/DD:[HH:MM]:<channel>:<ifghost>
			Calendar cal = Calendar.getInstance();
			int month = cal.get(Calendar.MONTH)+1;
			int day = cal.get(Calendar.DAY_OF_MONTH);
			int hour = cal.get(Calendar.HOUR_OF_DAY);
			int minute = cal.get(Calendar.MINUTE);
	    
			String sender = "BotCli:new:"+(month<10?"0":"")+month+"/"+(day<10?"0":"")+day+":["+(hour<10?"0":"")+hour+":"+(minute<10?"0":"")+minute+"]:"+channel+(ifghost==null?"":":"+ifghost);
	    
			Vector sstp_req = new Vector();
			sstp_req.add("SEND SSTP/1.4");
			sstp_req.add("Sender: "+sender);
			if (act.getIfGhostForSSTPServer() == null) {
				sstp_req.add("IfGhost: "+ghost_destination);
			}
			else {
				sstp_req.add("IfGhost: "+act.getIfGhostForSSTPServer());
			}
			sstp_req.add("Script: "+ScriptAnalyzer.expandYenURL(script));
	    
			Foundation.sharedFoundation().playSoundOfReceiveBottle();
			Foundation.sharedFoundation().getQueue().push(sstp_req);
		}

		if (act.getDrop() < 2) {
			LogDataServer.sharedLogServer().broadcastLog(script,channel,mid,uid,ghost_default,ifghost);
		}
    }
    
    public void docommand_unicastMessage(Vector param) {
		String uid = null,script = null;
        
		int n_params = param.size();
		for (int i = 0;i < n_params;i++) {
			String line = (String)param.elementAt(i);
			if (line.startsWith("SenderUID:")) {
				uid = line.substring(10).trim();
			}
			else if (line.startsWith("Script:")) {
				script = line.substring(7).trim();
			}
		}
	
		Vector sstp_req = new Vector();
		sstp_req.add("SEND SSTP/1.1");
		sstp_req.add("Sender: SSTP Bottle - unicasted message -");
		sstp_req.add("Script: "+ScriptAnalyzer.expandYenURL(script));
			
		Foundation.sharedFoundation().playSoundOfReceiveBottle();
		Foundation.sharedFoundation().getQueue().push(sstp_req);
        
		LogDataServer.sharedLogServer().unicastLog(script,uid);
	}
    
    public void docommand_forceBroadcastMessage(Vector param) {
		String mid = null,uid = null,msg = null,script = null;
		Vector sstp_req = null;
	
		int n_params = param.size();
		for (int i = 0;i < n_params;i++) {
			String line = (String)param.elementAt(i);
			if (line.startsWith("MID:")) {
				mid = line.substring(4).trim();
			}
			else if (line.startsWith("SenderUID:")) {
				uid = line.substring(10).trim();
			}
			else if (line.startsWith("DialogMessage:")) {
				msg = line.substring(14).trim();
			}
			else if (line.startsWith("Script:")) {
				script = line.substring(7).trim();
			}
		}
	
		if (script != null) {
			sstp_req = new Vector();
			sstp_req.add("SEND SSTP/1.1");
			sstp_req.add("Sender: SSTP Bottle - Message From Admin -");
			sstp_req.add("Script: "+ScriptAnalyzer.expandYenURL(script));
		}
	
		Foundation.sharedFoundation().playSoundOfReceiveBottle();
	
		if (msg != null) { // DialogMessageがあったら
			AdminMessageWindowController msgwin = new AdminMessageWindowController(msg);
			msgwin.show();
		}
		
		if (sstp_req != null) { // SSTPリクエストがあったら
			Foundation.sharedFoundation().getQueue().push(sstp_req);
		}
        
		LogDataServer.sharedLogServer().forceBroadcastLog(script,mid,uid,msg);
	}
    
    public void docommand_forceBroadcastInformation(Vector param) {
		String type = null,mid = null,num = null;
		
		int n_params = param.size();
		for (int i = 0;i < n_params;i++) {
			String line = (String)param.elementAt(i);
			if (line.startsWith("Type:")) {
				type = line.substring(5).trim();
			}
			else if (line.startsWith("MID:")) {
				mid = line.substring(4).trim();
			}
			else if (line.startsWith("Num:")) {
				num = line.substring(4).trim();
			}
		}
	
		try {
			if (type.equals("Vote")) {
				LogModifier.updateVoteNumOfBottle(mid,Integer.parseInt(num));
				//Foundation.sharedFoundation().getLogModifier().updateVoteNumOfBottle(mid,Integer.parseInt(num));
			}
			else if (type.equals("Agree")) {
				LogModifier.updateAgreeNumOfBottle(mid,Integer.parseInt(num));
				//Foundation.sharedFoundation().getLogModifier().updateAgreeNumOfBottle(mid,Integer.parseInt(num));
			}
		}
		catch (Exception e) {}
	}
    
    public void docommand_closeChannel(Vector param) {
		String ch_name = "";
	
		int n_params = param.size();
		for (int i = 0;i < n_params;i++) {
			String line = (String)param.get(i);
			if (line.startsWith("Channel:")) {
				ch_name = line.substring(8).trim();
			}
		}

		// chList から該當するチャンネルを取り除いて、MainWindow にリロー
		// ドさせる。
		for (Iterator ite = chList.iterator(); ite.hasNext(); ) {
			ChannelInfo info = (ChannelInfo)ite.next();

			if (info.name().equals(ch_name)) {
				ite.remove();
				break;
			}
		}

		mainWindow.reloadChListView();
	}
    
    public ChannelInfo findChInfoNamed(String name) {
		// 名前がnameであるチャンネルのChannelInfoを返します。
		// 見つからなかったらnullになります。
		if (name == null) return null;
		ChannelInfo result = null;
        
		int n_ch = chList.size();
		for (int i = 0;i < n_ch;i++) {
			ChannelInfo info = (ChannelInfo)chList.elementAt(i);
			if (info.name().equals(name)) {
				result = info;
				break;
			}
		}
        
		return result;
	}
    
    public boolean broadcast(String script,String chname,String ghost) {
		// sendBroadcastをポストします。
		// script: エンコード前のスクリプト
		// chname: チャンネル名
		// ghost: IfGhost指定（nullなら省略する）
		StringBuffer command = new StringBuffer();
		try {
			command.append("Command: sendBroadcast\r\n");
			command.append("Channel: "+chname+"\r\n");
			command.append("LUID: "+luid+"\r\n");
			command.append("Talk: "+script+"\r\n");
			if (ghost != null) command.append("Ghost: "+ghost+"\r\n");
		}
		catch (Exception e) {}
		// コマンド完成
        
		Hashtable resp = slppSession(command.toString());
		if (( (String)resp.get("Result") ).equals("Err")) {
			return false;
		}
		else {
			// その名前を持つチャンネルを探し、ゴースト名を取得。
			ChannelInfo ci = findChInfoNamed(chname);
			String ifghost = (ci == null ? null : ci.ghost());
        
			LogDataServer.sharedLogServer().sendBroadcastLog(script,chname,ifghost,ghost);
			return true;
		}
	}
    
    public void vote(String mid,String voteType) {
		// voteMessageをポストします。
		// voteTypeには文字列"Vote"又は"Agree"を渡して下さい。
		StringBuffer command = new StringBuffer();
		command.append("Command: voteMessage\r\n");
		command.append("LUID: "+luid+"\r\n");
		command.append("MID: "+mid+"\r\n");
		command.append("VoteType: "+voteType+"\r\n");
		
		slppSession(command.toString());
	}
    
    public Vector getChannelList() {
		return chList;
	}
    
    public void terminate() {
		signal_stop = true;
		while (isAlive()) {}
	}
}
