/*
	$Id: SurfaceDescription.java,v 1.1.1.1 2003/04/08 14:55:46 admin Exp $
	-----
	
    SSTP Bottle Client for MacOS X, using protocol SLPP2.0
    Copyright (C) 2001-2002 phonohawk.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package core;
import com.apple.cocoa.foundation.*;
import java.io.*;
import java.util.*;

public class SurfaceDescription
{
	protected static SurfaceDescription _shared_instance = null;
	public static final String home = NSPathUtilities.stringByExpandingTildeInPath("~");
	public static final String DEFAULT = "# Default #";
	
	Hashtable ghosts;

	public static SurfaceDescription sharedDescription()
	{
		if (_shared_instance == null) {
			_shared_instance = new SurfaceDescription(
				new File(home,"Library/BotCli/surface_description.txt"));
		}
		return _shared_instance;
	}
	
	protected SurfaceDescription(File f)
	{
		ghosts = new Hashtable();
		
		if (f.exists() && f.isFile()) {
			try {
				load(new FileInputStream(f));
			}
			catch (Exception e) {
				System.err.println("Couldn't read ~/Library/BotCli/surface_description.txt");
				//System.err.println(e.toString());
				e.printStackTrace();
			}
		}
		
		GhostInfo default_info = new GhostInfo(DEFAULT);
		default_info.setSakuraName("\u3055\u304f\u3089");
		default_info.addSakuraSurface(0,"0: \u30ce\u30fc\u30de\u30eb\u7acb\u3061\u7d75");
		default_info.addSakuraSurface(1,"1: \u30a8\u30ed\u9762");
		default_info.addSakuraSurface(2,"2: \u9a5a\u304d");
		default_info.addSakuraSurface(3,"3: \u4e0d\u5b89\u3001\u80f8\u3092\u96a0\u3059");
		default_info.addSakuraSurface(4,"4: \u306f\u3045\u301c");
		default_info.addSakuraSurface(5,"5: \u5fae\u7b11\u307f");
		default_info.addSakuraSurface(6,"6: \u76ee\u9589\u3058");
		default_info.addSakuraSurface(7,"7: \u6012\u308a");
		default_info.addSakuraSurface(8,"8: \u51b7\u7b11");
		default_info.addSakuraSurface(20,"20: \u770b\u677f");
		default_info.addSakuraSurface(22,"22: \u843d\u4e0b");
		default_info.setKeroName("\u3046\u306b\u3085\u3046");
		default_info.addKeroSurface(10,"10: \u901a\u5e38");
		default_info.addKeroSurface(11,"11: \u522e\u76ee");
		default_info.addKeroSurface(12,"12: \u843d\u4e0b");
		ghosts.put(DEFAULT,default_info);
	}
	
	public GhostInfo get(String ghost)
	{
		return (GhostInfo)ghosts.get(ghost);
	}
	
	protected void load(InputStream is) throws Exception
	{
		BufferedReader br = new BufferedReader(new InputStreamReader(is,"UTF-8"));
		
		GhostInfo current_ghost = null;
		int current_state = -1;		
		for (int line_count = 1;;line_count++) {
			String line = br.readLine();
			if (line == null) {
				break;
			}
			
			int comment_pos = line.indexOf("//");
			if (comment_pos != -1) {
				line = line.substring(0,comment_pos).trim();
			}
			else {
				line = line.trim();
			}
			
			if (line.length() == 0) {
				continue;
			}
			else if (line.equals("{")) {
				if (current_ghost == null) {
					throw new Exception("line "+line_count+": block without ghost name");
				}
			}
			else if (line.equals("}")) {
				if (current_ghost == null) {
					throw new Exception("line "+line_count+": block-end without ghost block");
				}
				else {
					if (current_state == 0 || current_state == 1) {
						current_state = -1;
					}
					else if (current_state == -1) {
						ghosts.put(current_ghost.getIfGhost(),current_ghost);
						current_ghost = null;
					}
				}
			}
			else {
				if (current_ghost == null) {
					int pos_colon = line.indexOf(':');
					if (pos_colon == -1) {
						throw new Exception("line "+line_count+": block definition without colon");
					}
					
					String block_id = line.substring(0,pos_colon).trim();
					String block_name = line.substring(pos_colon+1).trim();
					if (block_id.equals("ghost")) {
						current_ghost = new GhostInfo(block_name);
						current_state = -1;
					}
					else {
						throw new Exception("line "+line_count+": unknown block type");
					}
				}
				else {
					if (current_state == 0 || current_state == 1) {
						int pos_colon = line.indexOf(':');
						if (pos_colon == -1) {
							throw new Exception("line "+line_count+": surface definition without colon");
						}
						
						String key = line.substring(0,pos_colon).trim();
						String value = line.substring(pos_colon+1).trim();
						
						try {
							int id = Integer.parseInt(key);
							if (current_state == 0) {
								current_ghost.addSakuraSurface(id,key+": "+value);
							}
							else {
								current_ghost.addKeroSurface(id,key+": "+value);
							}
						}
						catch (NumberFormatException e) {
							throw new Exception("line "+line_count+": illegal surface id");
						}
					}
					else if (current_state == -1) {
						int pos_colon = line.indexOf(':');
						if (pos_colon == -1) {
							throw new Exception("line "+line_count+": scope definition without colon");
						}
						
						String scope_id = line.substring(0,pos_colon).trim();
						String scope_name = line.substring(pos_colon+1).trim();
						if (scope_id.equals("sakura")) {
							current_ghost.setSakuraName(scope_name);
							current_state = 0;
						}
						else if (scope_id.equals("kero")) {
							current_ghost.setKeroName(scope_name);
							current_state = 1;
						}
						else {
							throw new Exception("line "+line_count+": unknown scope type");
						}
					}
				}
			}
		}
	}
	
	public class GhostInfo
	{
		String ifghost;
		String sakura_name;
		String kero_name;
		Vector sakura_surfaces;
		Vector kero_surfaces;
		
		public GhostInfo(String ifghost)
		{
			this.ifghost = ifghost;
			sakura_surfaces = new Vector();
			kero_surfaces = new Vector();
		}
		
		public String getIfGhost() {
			return ifghost;
		}
		
		public GhostInfo setSakuraName(String name) {
			sakura_name = name;
			return this;
		}
		
		public String getSakuraName() {
			return sakura_name;
		}
		
		public GhostInfo setKeroName(String name) {
			kero_name = name;
			return this;
		}
		
		public String getKeroName() {
			return kero_name;
		}
		
		public GhostInfo addSakuraSurface(int id,String surface) {
			sakura_surfaces.add(new Object[] {new Integer(id),surface});
			return this;
		}
		
		public GhostInfo addKeroSurface(int id,String surface) {
			kero_surfaces.add(new Object[] {new Integer(id),surface});
			return this;
		}
		
		public Enumeration getSakuraSurfaces() {
			return sakura_surfaces.elements();
		}
		
		public Enumeration getKeroSurfaces() {
			return kero_surfaces.elements();
		}
	}
}
