/*
  $Id: LogBrowserListViewDataSource.java,v 1.3 2006/09/08 14:37:42 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package gui.logbrowser;
import gui.logbrowser.*;
import log.*;
import core.*;
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.util.*;

public class LogBrowserListViewDataSource implements NSTableView.DataSource {
    LogBrowserController controller;
    
    public LogBrowserListViewDataSource(LogBrowserController controller) {
		this.controller = controller;
    }
    
    // implementaion of NSTableView.DataSource
    public int numberOfRowsInTableView(NSTableView aTableView) {
        LogData holded = LogDataServer.sharedLogServer().getHolded();
        if (holded == null) return 0;
	
		Object[] group = controller.getSelectedGroup();
		if (group == null) return 0;
	
        return ((Vector)group[1]).size();
    }
    
    public boolean tableViewAcceptDrop(
        NSTableView tableView,
        NSDraggingInfo info,
        int row,
        int operation) {
        // optional
        return false;
    }
    
    public Object tableViewObjectValueForLocation(
        NSTableView aTableView,
        NSTableColumn aTableColumn,
        int rowIndex) {
        LogData holded = LogDataServer.sharedLogServer().getHolded();
		if (holded == null) return null;
	
		Object[] group = controller.getSelectedGroup();
		if (group == null) return null;
	
		if (rowIndex >= ((Vector)group[1]).size()) return null;
		
        Bottle info = (Bottle)((Vector)group[1]).elementAt(rowIndex);
        String column = (String)aTableColumn.identifier();
        
        if (column.equals("type")) {
			return null;
        }
        else if (column.equals("message")) {
			String type = "OB";
	    
			if (!type.equals("FB")) {
				// scriptのみ。
				String script = info.getScript();
				// 確かに分かり易いがこの名前は長すぎると思はないか。NSMu(略)ing
				NSMutableAttributedString attributed_script =
					(NSMutableAttributedString)info.getRemark("temporary-attributed-script");
		
				if (attributed_script == null) {
					// まだAttributed版が作られていない
					attributed_script = new NSMutableAttributedString(script);
					ScriptAnalyzer.analyze(attributed_script);
		    
					info.setRemark("temporary-attributed-script", attributed_script); // キャッシュ
				}
				return attributed_script;
			}
			else {
				// scriptのfbmsgどちらか又は両方。
				String script = info.getScript();
				String fbmsg = info.getFBMsg();
				NSMutableAttributedString attributed_script =
					(NSMutableAttributedString)info.getRemark("temporary-attributed-script");
				NSMutableAttributedString attributed_fbmsg = null;
		
				if (script != null) {
					if (attributed_script == null) {
						// まだAttributed版が作られていない
						attributed_script = new NSMutableAttributedString(script);
						ScriptAnalyzer.analyze(attributed_script);
			
						info.setRemark("temporary-attributed-script", attributed_script); // キャッシュ
					}
				}
				if (fbmsg != null) {
					attributed_fbmsg = new NSMutableAttributedString(fbmsg);
					attributed_fbmsg.addAttributeInRange(
						NSAttributedString.ForegroundColorAttributeName,
						NSColor.blueColor(),
						new NSRange(0,fbmsg.length())
						);
				}
		
				if (script == null && fbmsg != null) {
					return attributed_fbmsg;
				}
				else if (script != null && fbmsg == null) {
					return attributed_script;
				}
				else if (script != null && fbmsg != null) {
					// 連結
					attributed_fbmsg.appendAttributedString(new NSAttributedString(" ### "));
					attributed_fbmsg.appendAttributedString(attributed_script);
					return attributed_fbmsg;
				}
				else {
					return null;
				}
			}
        }
		else if (column.equals("ghost")) {
			// ifghostがあれば赤字でifghost。
			// 無ければ黒字でghost。
			String ifghost = info.getIfGhost();
			if (ifghost != null) {
				NSMutableAttributedString mas = new NSMutableAttributedString(ifghost);
				mas.addAttributeInRange(
					NSAttributedString.ForegroundColorAttributeName,
					NSColor.redColor(),
					new NSRange(0,ifghost.length())
					);
				return mas;
			}
	    
			return null; // チャンネル名から推測すべきだらうが…。ToDo。
		}
		else if (column.equals("v&a")) {
			// 整形：
			// ${agree}/${vote}
			int vote = info.getVote();
			int agree = info.getAgree();
			
			if (vote == 0 && agree == 0) {
				// どちらも存在しなければ空欄
				return null;
			}
			else {
				return agree + "/" + vote;
			}
		}
		else if (column.equals("time")) {
			String data = info.getTime();
			return data.substring(0,2)+':'+data.substring(2,4)+':'+data.substring(4);
		}
		else if (column.equals("channel")) {
			// channelはそのまま。
			return info.getChannel();
		}
		else {
			return null;
		}
    }
    
    public void tableViewSetObjectValueForLocation(
        NSTableView aTableView,
        Object anObject,
        NSTableColumn aTableColumn,
        int rowIndex) {
        // do nothing
    }
    
    public int tableViewValidateDrop(
        NSTableView tableView,
        NSDraggingInfo info,
        int row,
        int operation) {
        // optional
        return 0;
    }
    
    public boolean tableViewWriteRowsToPasteboard(
        NSTableView tableView,
        NSArray rows,
        NSPasteboard pboard) {
        // optional
        return false;
    }

    public void tableViewSortDescriptorsDidChange(
		NSTableView tableView, NSArray oldDescriptors) {
		// optional
    }

	public NSArray tableViewNamesOfPromisedFilesDroppedAtDestination(
		NSTableView tableView,
		java.net.URL url,
		NSIndexSet indexSet) {
		// optional
		return null;
	}

	public boolean tableViewWriteRowsToPasteboard(
		NSTableView tableView,
		NSIndexSet indexSet,
		NSPasteboard pboard) {
		// optional
		return false;
	}
}
