/*
  $Id: MainWindowController.java,v 1.3 2006/09/08 14:37:42 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package gui.main;
import gui.logbrowser.*;
import gui.logdownload.*;
import gui.main.*;
import gui.yenurl.*;
import core.*;
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.io.*;
import java.util.*;

public class MainWindowController extends NSWindowController
{
    private static MainWindowController _sharedInstance = null;
    
    ChannelInfoViewDataSource chInfoDataSource;
    MainWindowGhostComboDataSource scriptsend_combo_ghost_datasource;

    int bottle_length_limit;
    
    // outlets
    NSTableView channelListView;
    NSTextView scriptView;
    
    NSButton serverstats_btn_toggleconnect;
    NSTextField serverstats_channels;
    NSTextField serverstats_users;
    NSButton serverstats_connected;
    NSButton serverstats_receive;
    NSButton serverstats_send;

    NSTextField sstpqueue_inqueue;
    
    NSButton scriptsend_localtest;
    NSButton scriptsend_clear; // 表示されるボタン名はResetに変更しました。
    NSButton scriptsend_broadcast;
    NSComboBox scriptsend_combo_ghost;
	
    NSPopUpButton tags_s;
	
    NSTextField lengthofscript_current;
    NSTextField lengthofscript_limit;
    
    NSButton channel_login; // Join
    NSButton channel_logout; // Leave
    NSButton channel_selectall;
    NSTextField channel_registered;
    
    public static MainWindowController sharedInstance() {
        if (_sharedInstance == null) {
            _sharedInstance = new MainWindowController();
        }
        return _sharedInstance;
    }
    
    private MainWindowController() {
        super("MainWindow");
        loadWindow();
	
		window().setFrameUsingName("mainwindow_frame");
        window().setFrameAutosaveName("mainwindow_frame");
		scriptsend_clear(null);
        tableViewSelectionDidChange(null);
        textDidChange(null);
        
        chInfoDataSource = new ChannelInfoViewDataSource();
        channelListView.setDataSource(chInfoDataSource);
	
		scriptsend_combo_ghost_datasource = new MainWindowGhostComboDataSource();
		scriptsend_combo_ghost.setDataSource(scriptsend_combo_ghost_datasource);
		scriptsend_combo_ghost.selectItemAtIndex(0); // Default
		scriptsend_combo_ghost.setNumberOfVisibleItems(10);
		broadcasting_ghost_changed(null);
        
        String app_name = NSBundle.mainBundle().localizedStringForKey("CFBundleShortVersionString","SSTP Bottle Client for MacOS X","InfoPlist");
        window().setTitle(app_name);
	
		NSNotificationCenter.defaultCenter().addObserver(
			this,
			new NSSelector("notification_reload",new Class[] {NSNotification.class}),
			"BCGhostSpecModified",
			null);

		NSNotificationCenter.defaultCenter().addObserver(
			this,
			new NSSelector("notification_update_bottle_length_limit", new Class[] {NSNotification.class}),
			"BCBottleLengthLimitChanged",
			null);
	
		// リミット設定
		notification_update_bottle_length_limit(null);
	
		// スクリプト入力ビューをfirst responderにする。
		window().makeFirstResponder(scriptView);
    }

    public void setOpacity(float opacity) {
		// opacity: 0.0 - 1.0
		window().setAlphaValue(1.0f - opacity);
    }
    
    public void setConnectedState(boolean stat) {
        serverstats_connected.setState(stat ? NSCell.OnState : NSCell.OffState);
		serverstats_btn_toggleconnect.setTitle(stat ? "Disconnect" : "Connect");
	
		LogBrowserController.sharedLogBrowser().buttons_validate();
    }
    
    public void setReceiveState(boolean stat) {
        serverstats_receive.setState(stat ? NSCell.OnState : NSCell.OffState);
    }
    
    public void setSendState(boolean stat) {
        serverstats_send.setState(stat ? NSCell.OnState : NSCell.OffState);
    }
    
    public void setUsersCount(int c) {
        // -1が渡された場合は空白にします。
        if (c == -1) {
            serverstats_users.setStringValue("");
        }
        else {
            serverstats_users.setIntValue(c);
        }
    }
    
    public void setChannelsCount(int c) {
        // -1が渡された場合は空白にします。
        if (c == -1) {
            serverstats_channels.setStringValue("");
        }
        else {
            serverstats_channels.setIntValue(c);
        }
    }
    
    public void setChannelRegisteredCount(int c) {
        // -1が渡された場合は空白にします。
        if (c == -1) {
            channel_registered.setStringValue("");
        }
        else {
            channel_registered.setIntValue(c);
        }
    }
    
    public void setBottlesInQueueCount(int c) {
        // -1が渡された場合は空白にします。
        if (c == -1) {
            sstpqueue_inqueue.setStringValue("");
        }
        else {
            sstpqueue_inqueue.setIntValue(c);
        }
    }
    
    public void reloadChListView() {
        channelListView.reloadData();
        tableViewSelectionDidChange(null);
    }
    
    // notification handlers
    public void notification_reload(NSNotification notification) {
		scriptsend_combo_ghost_datasource.reloadFromDefaults();
		scriptsend_combo_ghost.reloadData();
    }

    public void notification_update_bottle_length_limit(NSNotification notification) {
		NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
		if (defaults.objectForKey("bottle-length-limit.limit") == null) {
			bottle_length_limit = 1900;
		}
		else {
			bottle_length_limit = defaults.integerForKey("bottle-length-limit.limit");
		}
		lengthofscript_limit.setStringValue(bottle_length_limit+" bytes");
    }
    
    // Actions
    public void channel_login(Object sender) {
        // 現在channelListViewで選択されているチャンネルを登録します。
        // 実際の登録処理はServerConnectionがSLPP2.0 CGIにコマンドをポストすることで行われます。
        if (!Foundation.sharedFoundation().slpp_connected()) return;
        ServerConnection sc = Foundation.sharedFoundation().getSlppConnection();
        Vector chList = sc.getChannelList();
        
        Vector channels = new Vector(); // String配列。中身はチャンネル名。
        int n_ch = channelListView.numberOfRows();
        for (int i = 0;i < n_ch;i++) {
            if (channelListView.isRowSelected(i)) {
                ChannelInfo info = (ChannelInfo)chList.elementAt(i);
                channels.add(info.name());
            }
        }
        
        sc.loginChannels(channels);
    }
    
    public void channel_logout(Object sender) {
        // 現在channelListViewで選択されているチャンネルの登録を解除します。
        // 実際の登録処理はServerConnectionがSLPP2.0 CGIにコマンドをポストすることで行われます。
        if (!Foundation.sharedFoundation().slpp_connected()) return;
        ServerConnection sc = Foundation.sharedFoundation().getSlppConnection();
        Vector chList = sc.getChannelList();
        
        Vector channels = new Vector(); // String配列。中身はチャンネル名。
        int n_ch = channelListView.numberOfRows();
        for (int i = 0;i < n_ch;i++) {
            if (channelListView.isRowSelected(i)) {
                ChannelInfo info = (ChannelInfo)chList.elementAt(i);
                channels.add(info.name());
            }
        }
        
        sc.logoutChannels(channels);
    }
    
    public void channel_selectall(Object sender) {
        channelListView.selectAll(null);
    }
    
    public void queue_sleep(NSButton sender) {
        SSTPQueue queueManager = Foundation.sharedFoundation().getQueue();
        queueManager.setSleeping(sender.state() == NSCell.OnState);
    }
	
    public void queue_clear(NSButton sender) {
		SSTPQueue queueManager = Foundation.sharedFoundation().getQueue();
		queueManager.clear();
    }
	
    public void broadcasting_ghost_changed(Object sender) {
		// senderは無視。
		String ifghost = scriptsend_combo_ghost.stringValue();
		SurfaceDescription.GhostInfo info = SurfaceDescription.sharedDescription().get(ifghost);
		if (info == null) {
			info = SurfaceDescription.sharedDescription().get(SurfaceDescription.DEFAULT);
		}
	
		NSMenu menu = tags_s.menu();
		menu.setAutoenablesItems(false);
		// まずは二つ目以降を全て削除
		final int n_items = menu.numberOfItems();
		for (int i = 1; i < n_items; i++) {
			menu.removeItemAtIndex(1);
		}
		// sakura側
		NSMenuItem sakura_label = new NSMenuItem(info.getSakuraName(),null,"");
		sakura_label.setEnabled(false);
		menu.addItem(sakura_label);
		for (Enumeration e = info.getSakuraSurfaces(); e.hasMoreElements(); ) {
			Object[] surface = (Object[])e.nextElement();
	    
			NSMenuItem item = new NSMenuItem((String)surface[1],null,"");
			item.setTag(((Integer)surface[0]).intValue());
			item.setEnabled(true);
	    
			menu.addItem(item);
		}
		// 仕切りを入れる
		menu.addItem((NSMenuItem)NSMenuItem.protocolSeparatorItem());
		// kero側
		NSMenuItem kero_label = new NSMenuItem(info.getKeroName(),null,"");
		kero_label.setEnabled(false);
		menu.addItem(kero_label);
		for (Enumeration e = info.getKeroSurfaces(); e.hasMoreElements(); ) {
			Object[] surface = (Object[])e.nextElement();
	    
			NSMenuItem item = new NSMenuItem((String)surface[1],null,"");
			item.setTag(((Integer)surface[0]).intValue());
			item.setEnabled(true);
	    
			menu.addItem(item);
		}
    }
	
    public static String getInitialScriptFormat() {
		NSUserDefaults defaults = NSUserDefaults.standardUserDefaults();
		String format = (String)defaults.objectForKey("iniscript.script");
		if (format == null) {
			return "\u00a5t\u00a5u\u00a5s[10]\u00a5h\u00a5s[0]|\u00a5e";
		}
		else {
			return format;
		}
    }
    
    public void scriptsend_clear(Object sender) {
		String format = getInitialScriptFormat();
	
		final int firstVerticalLinePos = format.indexOf('|');
		if (firstVerticalLinePos != -1) {
			// 縦線を全て消す
			StringBuffer sb = new StringBuffer(format);
			while (true) {
				int vertLinePos = sb.toString().indexOf('|');
				if (vertLinePos == -1) break;
		
				sb.deleteCharAt(vertLinePos);
			}
	    
			scriptView.setString(sb.toString());
			scriptView.setSelectedRange(new NSRange(firstVerticalLinePos,0));
		}
		else {
			scriptView.setString(format);
		}
		textDidChange(null);
    }
    
    public void scriptsend_localtest(Object sender) {
        String script = validateScript(scriptView.string());
        String app_name = NSBundle.mainBundle().localizedStringForKey("CFBundleShortVersionString","BotCli","InfoPlist");
        
		String ghost = null;
		if (channelListView.numberOfSelectedRows() == 1) {
            ServerConnection sc = Foundation.sharedFoundation().getSlppConnection();
            Vector chList = sc.getChannelList();
            int ch_id = channelListView.selectedRow();
            
            ghost = ((ChannelInfo)chList.elementAt(ch_id)).ghost();
        }
	
		String ifghost = scriptsend_combo_ghost.stringValue();
		if (ifghost.equals(MainWindowGhostComboDataSource.DEFAULT_ITEM)) {
			ifghost = null; // # Default #
		}
	
		String target = null;
		if (ifghost != null) target = ifghost;
		else if (ghost != null) target = ghost;
	
        Vector req = new Vector();
        req.add("SEND SSTP/1.4");
        req.add("Sender: "+app_name+" local-test;"+(target == null ? " " : " [Target:"+target+"]"));
        if (target != null) req.add("IfGhost: "+target);
        req.add("Script: "+ScriptAnalyzer.expandYenURL(script));
        req.add("Charset: SJIS");
        
        //Foundation.sharedFoundation().getQueue().push(req);
		Foundation.sharedFoundation().sendToLocalSstpd(req);
    }
    
    public void scriptsend_broadcast(Object sender) {
        if (channelListView.numberOfSelectedRows() != 1) return;
        
        ServerConnection sc = Foundation.sharedFoundation().getSlppConnection();
        Vector chList = sc.getChannelList();
        
        int ch_id = channelListView.selectedRow();
        
        String script = validateScript(scriptView.string());
        String chname = ((ChannelInfo)chList.elementAt(ch_id)).name();
		String ifghost = scriptsend_combo_ghost.stringValue();
		if (ifghost.equals(MainWindowGhostComboDataSource.DEFAULT_ITEM)) {
			ifghost = null; // # Default #
		}

		boolean succeeded = sc.broadcast(script,chname,ifghost);
		if (succeeded) {
			scriptsend_clear(null);
		}
    }
    
    private String validateScript(String param) {
        // 改行を￥nに置換し、￥eで終わっていなかったらそれを付ける。
        StringBuffer sb = new StringBuffer(param);
        int return_pos;
        while ((return_pos = sb.toString().indexOf('\n')) != -1) {
			sb.replace(return_pos,return_pos + 1,"\u00a5n");
		}
        if (!sb.toString().endsWith("\u00a5e")) sb.append("\u00a5e");
        
        return sb.toString();
    }

    public void serverstats_toggleconnect(Object sender) {
        if (Foundation.sharedFoundation().slpp_connected()) {
            //Foundation.sharedFoundation().slpp_disconnect(); // 切断
			new SlppTogglerThread(false).start();
        }
        else {
            //Foundation.sharedFoundation().slpp_connect(); // 接続
			new SlppTogglerThread(true).start();
        }
    }
    private class SlppTogglerThread extends Thread {
		// 押されたボタン
		boolean state_to_set;
		public SlppTogglerThread(boolean state_to_set) {
			this.state_to_set = state_to_set;
		}
		public void run() {
			int pool = NSAutoreleasePool.push();
			if (state_to_set) {
				Foundation.sharedFoundation().slpp_connect(); // 接続
			}
			else {
				Foundation.sharedFoundation().slpp_disconnect(); // 切断
			}
			NSAutoreleasePool.pop(pool);
		}
    }
    
    public void tags_s(NSPopUpButton sender) {
        scriptView.insertText("\u00a5s["+sender.selectedItem().tag()+']');
    }
    
    public void tags_simple(NSButton sender) {
        scriptView.insertText(sender.title());
    }
    
    public void tags_url(Object sender) {
		YenURLAssistantController.sharedController().showWindow(null);
    }
    
    public void tags_w(NSPopUpButton sender) {
        scriptView.insertText("\u00a5w"+sender.selectedItem().tag());
    }
    
    public void tags_percent(NSPopUpButton sender) {
        String title = sender.selectedItem().title();
        int sp_pos;
        if ((sp_pos = title.indexOf(' ')) == -1) return;
        
        scriptView.insertText(title.substring(0,sp_pos));
    }
    
    public void logwindow(Object sender) {
        LogBrowserController lbc = LogBrowserController.sharedLogBrowser();
        lbc.showWindow(null);
    }
    
    public void logdownload(Object sender) {
		LogDownloadController.sharedController().showWindow(null);
    }
    
    public void insertTextToScriptView(String str) {
		scriptView.insertText(str);
    }
    
    // deligate
    public void tableViewSelectionDidChange(NSNotification notification) {
        // ユーザーがチャンネルリスト内の項目を選択すると呼ばれます。
        // notificationは無視します。
        
        // 選択されている項目が一つもなかったらlogin/outボタンはdisable。
        // 未登録のチャンネルが一つでも選択されていたらloginボタンをenable。そうでなければdisable。
        // 登録中のチェンネルが一つでもあったらlogoutボタンをenable。そうでなければdisable。
        
        // 項目が一つ以上あればSelectAllボタンをenable。
        channel_selectall.setEnabled(channelListView.numberOfRows() > 0);
		/*System.out.println("getting num of rows");
		  int num_of_channels_tmp = channelListView.numberOfRows();
		  System.out.println("num of rows:"+num_of_channels_tmp);
		  channel_selectall.setEnabled(num_of_channels_tmp > 0);
		  System.out.println("set enabled of selectall to "+(num_of_channels_tmp > 0));*/
        
        textDidChange(null);
        
        if (channelListView.numberOfSelectedRows() == 0 || !Foundation.sharedFoundation().slpp_connected()) {
            channel_login.setEnabled(false);
            channel_logout.setEnabled(false);
            return;
        }
        
        boolean regd_ch_found = false;
        boolean unregd_ch_found = false;
        
        int n_ch = channelListView.numberOfRows();
        
        Vector chList = Foundation.sharedFoundation().getSlppConnection().getChannelList();
        for (int i = 0;i < n_ch;i++) {
            if (channelListView.isRowSelected(i)) {
                ChannelInfo info = (ChannelInfo)chList.elementAt(i);
                
                if (info.registered()) {
                    regd_ch_found = true;
                }
                else {
                    unregd_ch_found = true;
                }
                
                if (regd_ch_found && unregd_ch_found) {
                    break; // これ以上のループは無駄。
                }
            }
        }
        
        channel_logout.setEnabled(regd_ch_found);
        channel_login.setEnabled(unregd_ch_found);
    }
    
    public void textDidChange(NSNotification aNotification) {
        // SakuraScript構文チェックを行い、色分けする。
        // notificationは無視します。
        
        String data = scriptView.string();
        int len = data.length();
        boolean error = ScriptAnalyzer.analyze(scriptView);
        
        // 接続されていて文字が入力されていて構文エラーが無くてログイン中のチャンネルを一つだけ選択している状態なら
        // Broadcastボタンをenable。ただし選択されているチャンネルがNoPostだったらその限りではない。
        // マルチポストは禁止とする。
        if (data.length() > 0 && !error && channelListView.numberOfSelectedRows() == 1 && Foundation.sharedFoundation().slpp_connected()) {
            int selected = channelListView.selectedRow();
            Vector chList = Foundation.sharedFoundation().getSlppConnection().getChannelList();
            ChannelInfo info = (ChannelInfo)chList.elementAt(selected);
            if (info.registered() && !info.nopost()) {
				scriptsend_broadcast.setEnabled(true);
            }
            else {
                scriptsend_broadcast.setEnabled(false);
            }
        }
        else {
            scriptsend_broadcast.setEnabled(false);
        }
        
        // 文字が入力されていて構文エラーが無くて￥eで終わっていたらLocalTestボタンをenable。
        scriptsend_localtest.setEnabled(data.length() > 0 && !error);
        
        // 中身が初期スクリプトでなければClearボタンをenable。
        //scriptsend_clear.setEnabled(!data.equals("\u00a5t\u00a5h\u00a5s[0]"));
		// やめ。常にenabled。
	
		// サイズ算出
		try {
			int length_of_script = data.getBytes("SJIS").length;
			lengthofscript_current.setStringValue(length_of_script == 1 ? "1 byte" : length_of_script+" bytes");
	    
			if (length_of_script > bottle_length_limit) {
				scriptsend_broadcast.setEnabled(false);
			}
		} catch (Exception e) {}
    }
}
