/*
  $Id: YenURLAssistantURLTableContent.java,v 1.2 2006/09/08 14:37:42 admin Exp $
  -----
	
  SSTP Bottle Client for MacOS X, using protocol SLPP2.0
  Copyright (C) 2001-2002 phonohawk.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package gui.yenurl;
import gui.yenurl.*;
import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.util.*;

public class YenURLAssistantURLTableContent implements NSTableView.DataSource {
    Vector content; // 中身はYenURLAssistantURLTableContent.TableElement。
    NSTableView table;
    YenURLAssistantTableStyleView tableStyleView;
    
    public YenURLAssistantURLTableContent(YenURLAssistantTableStyleView tableStyleView,NSTableView table) {
		this.tableStyleView = tableStyleView;
		this.table = table;
		content = new Vector();
	}
    
    public void addElement() {
		int index = table.selectedRow();
		if (index == -1) { // どこも選択されていない
			content.add(new TableElement());
			table.reloadData();
			table.selectRow(content.size()-1,false);
		}
		else {
			content.add(index,new TableElement());
			table.reloadData();
			table.selectRow(index,false);
		}
		tableStyleView.validate();
	}
    
    public void removeSelectedElement() {
		int index = table.selectedRow();
		if (index == -1) return;
		
		content.remove(index);
		table.reloadData();
		tableStyleView.validate();
	}
    
    public void moveUpSelection() {
		int index = table.selectedRow();
		if (index == -1 || index == 0) return;
		
		Object temp = content.elementAt(index);
		content.remove(index);
		content.add(index-1,temp);
		table.reloadData();
		table.selectRow(index-1,false);
	}
    
    public void moveDownSelection() {
		int index = table.selectedRow();
		if (index == -1 || index == content.size()-1) return;
		
		Object temp = content.elementAt(index);
		content.remove(index);
		content.add(index+1,temp);
		table.reloadData();
		table.selectRow(index+1,false);
	}
    
    public boolean checkIfAllGreen() {
		int n_elems = content.size();
		for (int i = 0;i < n_elems;i++) {
			TableElement elem = (TableElement)content.elementAt(i);
			if (elem.getLabel().length() == 0 || !elem.getURL().startsWith("http://")) return false;
		}
		return true;
	}
    
    public void removeAll() {
		content.clear();
		table.reloadData();
		tableStyleView.validate();
	}
    
    public String generateTablePartOfYenURLTag() {
		// \URL[cancel][URL1][LABEL1][URL2][LABEL2]...
		//             ^ここからの範囲のタグを作ります。
		StringBuffer buf = new StringBuffer();
		int n_elems = content.size();
		for (int i = 0;i < n_elems;i++) {
			TableElement elem = (TableElement)content.elementAt(i);
			buf.append('['+elem.getURL()+"]["+elem.getLabel()+']');
		}
		return buf.toString();
	}
    
    public int numberOfRowsInTableView(NSTableView aTableView) {
		return content.size();
	}
    
    public boolean tableViewAcceptDrop(
        NSTableView tableView,
        NSDraggingInfo info,
        int row,
        int operation) {
		// optional
		return false;
	}
    
    public Object tableViewObjectValueForLocation(
        NSTableView aTableView,
        NSTableColumn aTableColumn,
        int rowIndex) {
		TableElement elem = (TableElement)content.elementAt(rowIndex);
		String column = (String)aTableColumn.identifier();
        
		if (column.equals("Label")) {
			return elem.getLabel();
		}
		else if (column.equals("URL")) {
			return elem.getURL();
		}
		else {
			return null;
		}
	}
    
    public void tableViewSetObjectValueForLocation(
        NSTableView aTableView,
        Object anObject,
        NSTableColumn aTableColumn,
        int rowIndex) {
		TableElement elem = (TableElement)content.elementAt(rowIndex);
		String column = (String)aTableColumn.identifier();
        
		if (column.equals("Label")) {
			elem.setLabel((String)anObject);
		}
		else if (column.equals("URL")) {
			elem.setURL((String)anObject);
		}
		tableStyleView.validate();
	}
    
    public int tableViewValidateDrop(
        NSTableView tableView,
        NSDraggingInfo info,
        int row,
        int operation) {
		// optional
		return 0;
	}
    
    public boolean tableViewWriteRowsToPasteboard(
        NSTableView tableView,
        NSArray rows,
        NSPasteboard pboard) {
		// optional
		return false;
	}
    
    private class TableElement {
		String label;
		String url;
		
		public TableElement() {
			label = "";
			url = "";
		}
		
		public TableElement(String label,String url) {
			this.label = label;
			this.url = url;
		}
		
		public String getLabel() {
			return label;
		}
		
		public void setLabel(String label) {
			this.label = label;
		}
		
		public String getURL() {
			return url;
		}
		
		public void setURL(String url) {
			this.url = url;
		}
    }

    public void tableViewSortDescriptorsDidChange(
		NSTableView tableView, NSArray oldDescriptors) {
		// optional
    }

	public NSArray tableViewNamesOfPromisedFilesDroppedAtDestination(
		NSTableView tableView,
		java.net.URL url,
		NSIndexSet indexSet) {
		// optional
		return null;
	}

	public boolean tableViewWriteRowsToPasteboard(
		NSTableView tableView,
		NSIndexSet indexSet,
		NSPasteboard pboard) {
		// optional
		return false;
	}
}
