/*  $Id: LogDataServer.java,v 1.5 2006/09/08 14:37:42 admin Exp $
	
    SSTP Bottle Client for MacOS X, using protocol SLPP2.0
    Copyright (C) 2001-2004 phonohawk.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package log;
import log.*;
import com.apple.cocoa.foundation.*;
import gui.logbrowser.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.util.regex.*;
import java.net.*;

/*
  ToDo:

  JavaVMのメモリ状況を見てキャッシュを管理する。好い加減なメモリ管理は事故の元。
*/

public class LogDataServer extends Thread {
    protected static LogDataServer _sharedInstance = null;
    public static String home = NSPathUtilities.stringByExpandingTildeInPath("~");
    public static final int CACHE_SIZE = 7;
    public static final int AUTO_COMMIT_INTERVAL = 120 * 1000; // 2分毎に自動commit
    
    LogData holded = null; // cacheと違い、追い出される事が無いもの。
    Vector cache; // <LogData>
    
    public static LogDataServer sharedLogServer() {
        if (_sharedInstance == null) {
            _sharedInstance = new LogDataServer();
			_sharedInstance.start();
        }
        return _sharedInstance;
    }
    
    protected LogDataServer() {
        cache = new Vector();
    }
	
    public void run() {
		while (true) {
			try {
				sleep(AUTO_COMMIT_INTERVAL);
			} catch (Exception e) {}
			
			commitAll();
		}
    }
	
    public LogData hold(int year,int month,int day) {
		unhold();
		holded = get(year,month,day);
		return holded;
    }
    
    public void unhold() {
		if (holded != null) {
			holded.commit();
			System.gc();
		}
		holded = null;
    }
    
    public LogData getHolded() {
		return holded;
    }
    
    public LogData get(int year,int month,int day) {
		// holdされていたら、それを返す。
		if (holded != null &&
			holded.getYear() == year &&
			holded.getMonth() == month &&
			holded.getDay() == day) {
	    
			return holded;
		}
	
		// cacheに入っていたら、それを返す。
		for (Enumeration e = cache.elements(); e.hasMoreElements(); ) {
			LogData log = (LogData)e.nextElement();
			if (log.getYear() == year &&
				log.getMonth() == month &&
				log.getDay() == day) {
		
				return log;
			}
		}
	
		// 入っていない。
		LogData new_log = new LogData(year,month,day);
		cacheThis(new_log);
		return new_log;
    }
    
    protected void cacheThis(LogData log) {
		// cacheに入っていたら、それを先頭に移動。
		// 入っていなければ、キャッシュが一杯なら最後の項目を消して先頭に追加。
		int cached_pos = cache.indexOf(log);
		if (cached_pos == -1) {
			if (cache.size() >= CACHE_SIZE) {
				// 消す前にcommitする。
				LogData last = (LogData)cache.elementAt(cache.size()-1);
				last.commit();
				cache.remove(cache.size()-1);
				System.gc();
			}
		}
		else {
			cache.remove(cached_pos);
			System.gc();
		}
		cache.add(0,log);
    }
    
    public void commitAll() {
		// キャッシュされている全てのログをcommitする。
		if (holded != null) {
			holded.commit();
		}
	
		synchronized (cache) {
			for (Enumeration e = cache.elements(); e.hasMoreElements(); ) {
				LogData log = (LogData)e.nextElement();
				log.commit();
			}
		}
    }
	
    public void broadcastLog(String script,String channel,String mid,String uid,String ghost,String ifghost) {
        submit(LogData.LOG_ALL,script,channel,null,mid,uid,ghost,ifghost);
    }
    
    public void sendBroadcastLog(String script,String channel,String ghost,String ifghost) {
        submit(LogData.LOG_USER,script,channel,null,null,null,ghost,ifghost);
    }
    
    public void unicastLog(String script,String uid) {
        // とうとう仕様書からも消えた。
    }
    
    public void forceBroadcastLog(String script,String mid,String uid,String message) {
		submit(LogData.LOG_ADMIN,script,null,message,mid,uid,null,null);
    }
    
    private void submit(int type, String script, String channel, String fbmsg,
						String mid, String uid, String ghost, String ifghost) {
        // 今日の日付を取得して、そのファイルが無かったら作る
        // パラメータは、nullの部分は省略されたものと見なす。
        // エンコードはしないで渡してください。
		Calendar cal = Calendar.getInstance();
		int year = cal.get(Calendar.YEAR);
		int month = cal.get(Calendar.MONTH) + 1;
		int day = cal.get(Calendar.DAY_OF_MONTH);
		int hour = cal.get(Calendar.HOUR_OF_DAY);
		int minute = cal.get(Calendar.MINUTE);
		int second = cal.get(Calendar.SECOND);
	
		// 今日のログを取得
		LogData log_of_today = get(year,month,day);
        
        /*
		  logファイルの形式：
		  type:[タイプ],script:[スクリプト],...
		  のように、ラベル:データ,ラベル:データ,...という状態で重ねていき、データ一塊について一行となる。
		  こうすれば必要無いフィールドは省略して良いし、将来ログファイルの形式が変わっても対応できる。
          
          
		  ラベル：
		  type: UB / OB / UU / OU / FB のどれか。それぞれ、User Broadcasted / Others Broadcasted / User Unicasted / Others Unicasted / ForceBroadcasted。後のプロトコルではunicastが自分でもできるようになりそうだから。
		  script: ShiftJIS + www-form-url-encodedでエンコードされたSakuraScript
		  channel: scriptと同じエンコードでチャンネル名
		  mid: エンコード必要ないようだが、念のためwww-form-url-encodedでのMID
		  date: MMDDYYYY 数値
		  time: HHMMSS 24時間表記
		  fbmsg: scriptと同じエンコードでForceBroadcastのダイアログメッセージ
		  uid: エンコード必要ないようだが、念のためwww-form-url-encodedでのUID
		  ghost: scriptと同じエンコードでゴースト名
		  ifghost: scriptと同じエンコードでIfGhost（送信時ではGhost）の指定
		  vote: 数値
		  agree: 数値
        */
        Bottle bottle = new Bottle();
		bottle.setScript(script);
		bottle.setChannel(channel);
		bottle.setMID(mid);
		bottle.setDate((month <= 9 ? "0"+month : ""+month) + (day <= 9 ? "0"+day : ""+day) + year);
		bottle.setTime((hour <= 9 ? "0"+hour : ""+hour) + (minute <= 9 ? "0"+minute : ""+minute) + (second <= 9 ? "0"+second : ""+second));
		bottle.setFBMsg(fbmsg);
		// bottle.put("uid",uid); 廢止された。
		// bottle.put("ghost",ghost); これはBottleが持つ必要が無い。
		bottle.setIfGhost(ifghost);
	
		// 追加
		log_of_today.add(type, bottle);
	
		// もしブラウザで表示中のファイルが今日のものだったら、リロードする。
		String pathOfBrowser = "/"+year+"/"+(month <= 9 ? "0"+month : ""+month)+"/"+(day <= 9 ? "0"+day : ""+day);
        LogBrowserController logBrowser = LogBrowserController.sharedLogBrowser();
        if (logBrowser.browserPath().equals(pathOfBrowser)) {
			logBrowser.reloadTable();
		}
    }
    
    public static Vector logged_years() {
		// 戻り値: Vector<String>
		Vector result = new Vector();
		
		File parentdir = new File(home,"Library/BotCli/log");
        if (!parentdir.exists()) {
			return result;
		}
		
		// 年のディレクトリ名として許されるもの
		Pattern pat = Pattern.compile("^\\d+$");
        
        File[] files = parentdir.listFiles();
		for (int i = 0; i < files.length; i++) {
			File f = files[i];
			
			if (f.isDirectory() && pat.matcher(f.getName()).matches()) {
				result.add(f.getName());
			}
		}
		
		return result;
    }
	
    public static Vector logged_months(String year) {
		// 戻り値: Vector<String>
		Vector result = new Vector();
		
		File parentdir = new File(home,"Library/BotCli/log/"+year);
        if (!parentdir.exists()) {
			return result;
		}
		
		// 月のディレクトリ名として許されるもの
		Pattern pat = Pattern.compile("^\\d+$");
        
        File[] files = parentdir.listFiles();
		for (int i = 0; i < files.length; i++) {
			File f = files[i];
			
			if (f.isDirectory() && pat.matcher(f.getName()).matches()) {
				result.add(f.getName());
			}
		}
		
		return result;
    }
	
    public static Vector logged_days(String year,String month) {
		// 戻り値: Vector<String>
		Vector result = new Vector();
		
		File parentdir = new File(home,"Library/BotCli/log/"+year+"/"+month);
        if (!parentdir.exists()) {
			return result;
		}
		
		// 01.log
		// 01.log.gz
		// 01.xml
		// 01.xml.gz
		// 01-extra.xml
		// 01-extra.xml.gz
		// どの形式でも良いが、重複があれば削除される。
		
		String gz = "(?:\\.gz)";
		Pattern pat = Pattern.compile("^(\\d+)(?:\\.log"+gz+"?|(?:-extra)?\\.xml"+gz+"?)$");
        
        File[] files = parentdir.listFiles();
		for (int i = 0; i < files.length; i++) {
			File f = files[i];
			
			if (f.isFile()) {
				Matcher matcher = pat.matcher(f.getName());
				if (matcher.find()) {
					result.add(matcher.group(1));
				}
			}
		}
		
		Vector uniq = new Vector();
		String last = null;
		for (Enumeration e = result.elements(); e.hasMoreElements(); )
			{
				String current = (String)e.nextElement();
			
				if (!current.equals(last)) {
					uniq.add(current);
					last = current;
				}
			}
		
		return uniq;
    }
}
